﻿<#

.SYNOPSIS
This Powershell script can be run on a server to gather a file collection from a NetBackup instance.
The generated files can be uploaded to LiveOpics through the collector for analysis. 

.DESCRIPTION
This script will run queries against NetBackup and produce .out files for analysis.

.LINK
https://www.liveoptics.com/

#>

#-----------------------------------------------------------------------------
#  Copyright (c) 2017-2018 by Dell Inc. 
# 
#  All rights reserved.  
# 
#-----------------------------------------------------------------------------

$scriptVersion="1.0.011";

""
"------------------------------------------------------";
"Live Optics collection script for NetBackup on Windows";
"------------------------------------------------------";
$date=Get-Date -Format o;
"Date: " + $date;
"Version: " + $scriptVersion;
""

#
# UTF-8 output to file without BOM
#
function Out-FileUtf8NoBom {

  [CmdletBinding()]
  param(
    [Parameter(Mandatory=$True, Position=0)] [string] $LiteralPath,
    [switch] $Append,
    [switch] $NoClobber,
    [Parameter(ValueFromPipeline=$True)]
    [string[]]$lines
  )

  Begin
  {
	# Make sure that the .NET framework sees the same working dir. as PS
	# and resolve the input path to a full path.
	[System.IO.Directory]::SetCurrentDirectory($PWD) # Caveat: .NET Core doesn't support [Environment]::CurrentDirectory
	$LiteralPath = [IO.Path]::GetFullPath($LiteralPath)

	# If -NoClobber was specified, throw an exception if the target file already
	# exists.
	if ($NoClobber -and (Test-Path $LiteralPath)) {
		Throw [IO.IOException] "The file '$LiteralPath' already exists."
	}

	try {
		
		# Create a StreamWriter object.
		# Note that we take advantage of the fact that the StreamWriter class by default:
		# - uses UTF-8 encoding
		# - without a BOM.
		$sw = New-Object IO.StreamWriter $LiteralPath, $Append

	}
	catch {
		Write-Error "Failed to create output stream."
		Write-Error $_;
		exit;
    }	
  }
  
  Process { 
	try {
		ForEach ($line in $lines) {
			$sw.WriteLine($line)
		}
	}
	catch {
		$sw.Dispose();
		$sw = $null;
		
		Write-Error "Failed to write to stream."
		Write-Error $_;
		exit;
	}
  }
  
  End {
	if ($sw) {
		$sw.Dispose();
	}
  }
}


#
# Create a temporary directory
#
$dt=Get-Date -UFormat %Y%m%d%H%M%S
$tempDir="LiveOpticsNetBackup" + $dt
new-item -type directory -path $tempDir | Out-Null;

if (!(Test-Path $tempDir)) {
    
    Write-Error "Failed to create directory to hold file output: $tempDir";
    exit;
}

"Temporary directory created: " + $tempDir;

#
# Set the path
#
$env:Path="$env:Path;$env:ProgramFiles\veritas\netbackup\bin\admincmd;$env:ProgramFiles\veritas\netbackup\bin;$env:ProgramFiles\veritas\volmgr\bin";

#
# Date information - quick_date.out
#
$quick_date="$tempDir/quick_date.out";
$date=Get-Date -Uformat "%a %m/%d/%Y"
$date | Out-FileUtf8NoBom $quick_date;

#
# Improved date format
#
$quick_psdate="$tempDir/quick_psdate.out";
$date=Get-Date -Format o;
$date | Out-FileUtf8NoBom $quick_psdate;

#
# Script version
#
$quick_scriptVersion="$tempDir/quick_scriptVersion.out";
$scriptVersion | Out-FileUtf8NoBom $quick_scriptVersion;

#
#Windows version
#
$quick_osVersion="$tempDir/quick_windowsversion.out";
[environment]::OSVersion.Version | Out-FileUtf8NoBom $quick_osVersion

#
#Powershell version
#
$quick_psVersion="$tempDir/quick_psversion.out";
$PSVersionTable.PSVersion | Out-FileUtf8NoBom $quick_psVersion

#
# Time information
#
$startTime=[System.DateTime]::Now;
$startTime.ToString("hh tt", [System.Globalization.CultureInfo]::InvariantCulture.DateTimeFormat) | Out-FileUtf8NoBom "$tempDir/quick_time.out";

#
# Timezone information - quick_timezone.out
#
$tz=(& reg query "HKLM\SYSTEM\CurrentControlSet\Control\TimeZoneInformation");
$quick_timezone="$tempDir/quick_timezone.out";
$tz | Out-FileUtf8NoBom $quick_timezone;

#
# Hostname information - quick_hostname.out
#
$hostnameFile="$tempDir/quick_hostname.out";
hostname | Out-FileUtf8NoBom $hostnameFile;

#
# Base date used for drawing the line on backups.
# 120 days is the default.
#
$last=(New-TimeSpan -Days (120));
$lastBackupDate=[System.DateTime]::UtcNow.Subtract($last);
$lastBackupDateString=$lastBackupDate.ToString("MM\/dd\/yyyy");

#
# Last Backup Date that we care about - quick_nbbasedate.out
#
$quick_nbbasedate="$tempDir/quick_nbbasedate.out";
$lastBackupDateString | Out-FileUtf8NoBom $quick_nbbasedate;

#
# Dump environment variables
#
foreach ($e in (gci env:*)) { $e.Name + "=" + $e.Value | Out-FileUtf8NoBom "$tempDir/quick_set.out" -Append; }

#
# quick_bpconfig.out
#
"Running command: bpconfig -L -U > $tempDir/quick_bpconfig.out";

& bpconfig -L -U | Out-FileUtf8NoBom "$tempDir/quick_bpconfig.out";

#
# quick_bpgetconfig.out
#
"Running command: bpgetconfig -X > $tempDir/quick_bpgetconfig.out";

& bpgetconfig -X | Out-FileUtf8NoBom "$tempDir/quick_bpgetconfig.out";

#
# quick_bpminlicense.out
#
"Running command: bpminlicense -list_keys -verbose > $tempDir/quick_bpminlicense.out";

& bpminlicense -list_keys -verbose | Out-FileUtf8NoBom "$tempDir/quick_bpminlicense.out";

#
# quick_bpretlevel.out
#
"Running command: bpretlevel > $tempDir/quick_bpretlevel.out";

& bpretlevel | Out-FileUtf8NoBom "$tempDir/quick_bpretlevel.out";

#
# quick_bpstulist.out
#
"Running command: bpstulist -U -L > $tempDir/quick_bpstulist.out";

& bpstulist -U -L | Out-FileUtf8NoBom "$tempDir/quick_bpstulist.out";

#
# quick_bpstulist_go.out
#
"Running command: bpstulist -U -go -L > $tempDir/quick_bpstulist_go.out";
& bpstulist -U -go -L | Out-FileUtf8NoBom "$tempDir/quick_bpstulist_go.out";

#
# quick_bpclient.out
#
"Running command: bpclient -All -L > $tempDir/quick_bpclient.out";
& bpclient -All -L | Out-FileUtf8NoBom "$tempDir/quick_bpclient.out";

#
# quick_bpplclients.out
#
"Running command: bpplclients > $tempDir/quick_bpplclients.out";
& bpplclients | Out-FileUtf8NoBom "$tempDir/quick_bpplclients.out";

#
# quick_bppllist.out
#
"Running command: bppllist -allpolicies -L -U > $tempDir/quick_bppllist.out";
& bppllist -allpolicies -L -U | Out-FileUtf8NoBom "$tempDir/quick_bppllist.out";

#
# quick_bppllistl.out
#
"Running command: bppllist -allpolicies -l > $tempDir/quick_bppllistl.out";
& bppllist -allpolicies -l > "$tempDir/quick_bppllistl.out";

#
# quick_bpimagelist_err.out
# quick_bpimagelist.out
#
"Running command: bpimagelist -U -force -L -d $lastBackupDateString 2> $errorFile > $tempDir/quick_bpimagelist.out";
$errorFile="$tempDir/quick_bpimagelist_err.out";
& bpimagelist -U -force -L -d $lastBackupDateString 2> $errorFile | Out-FileUtf8NoBom "$tempDir/quick_bpimagelist.out";

#
# Check for errors
#
if ((test-path $errorFile) -and ((get-childitem $errorFile).Length -gt 0)) {

    #
    # Command had some errors.
    # Don't use the force command
    #
    move-item -Path $errorFile -Destination "$tempDir/quick_bpimagelist_force_err.out";
    
    "Running command: bpimagelist -U -L -d $lastBackupDateString 2> $errorFile > $tempDir/quick_bpimagelist.out";
    & bpimagelist -U -L -d $lastBackupDateString 2> $errorFile | Out-FileUtf8NoBom "$tempDir/quick_bpimagelist.out";
}

#
# quick_bpimmedial.out
# quick_bpimmedial_err.out
#
$errorFile="$tempDir/quick_bpimmedial_err.out";
"Running command: bpimmedia -U -force -l -d $lastBackupDateString 2> $errorFile > $tempDir/quick_bpimmedial.out";
& bpimmedia -U -force -l -d $lastBackupDateString 2> $errorFile | Out-FileUtf8NoBom "$tempDir/quick_bpimmedial.out";

#
# Check for errors
#
if ((test-path $errorFile) -and ((get-childitem $errorFile).Length -gt 0)) {

    #
    # Command had some errors.
    # Don't use the force command
    #
    move-item -Path $errorFile -Destination "$tempDir/quick_bpimmedial_force_err.out";
    
    "Running command: bpimmedia -U -l -d $lastBackupDateString 2> $errorFile > $tempDir/quick_bpimmedial.out";
    & bpimmedia -U -l -d $lastBackupDateString 2> $errorFile | Out-FileUtf8NoBom "$tempDir/quick_bpimmedial.out";
}

#
# quick_bpmedialist.out
#
"Running command: bpmedialist -mlist -U -L > $tempDir/quick_bpmedialist.out";
& bpmedialist -mlist -U -L | Out-FileUtf8NoBom "$tempDir/quick_bpmedialist.out";

#
# quick_bpmedialistsummary.out
#
"Running command: bpmedialist -summary -U -L > $tempDir/quick_bpmedialistsummary.out";
& bpmedialist -summary -U -L | Out-FileUtf8NoBom "$tempDir/quick_bpmedialistsummary.out";

#
# quick_vmquery.out
#
"Running command: vmquery -a > $tempDir/quick_vmquery.out"
& vmquery -a | Out-FileUtf8NoBom "$tempDir/quick_vmquery.out";

#
# quick_tpconfig.out
#
"Running command: tpconfig -dl > $tempDir/quick_tpconfig.out";
& tpconfig -dl | Out-FileUtf8NoBom "$tempDir/quick_tpconfig.out";

#
# quick_tpconfig_virtualmachines.out
#
"Running command: tpconfig -dvirtualmachines > $tempDir/quick_tpconfig_virtualmachines.out";
& tpconfig -dvirtualmachines | Out-FileUtf8NoBom "$tempDir/quick_tpconfig_virtualmachines.out";

#
# quick_tpconfig_diskarrays.out
#
"Running command: tpconfig -ddiskarrays > $tempDir/quick_tpconfig_diskarrays.out";
& tpconfig -ddiskarrays | Out-FileUtf8NoBom "$tempDir/quick_tpconfig_diskarrays.out";

#
# quick_vmglob.out
#
"Running command: vmglob -listall > $tempDir/quick_vmglob.out";
& vmglob -listall | Out-FileUtf8NoBom "$tempDir/quick_vmglob.out";

#
# quick_vmpool.out
#
"Running command: vmpool -listall > $tempDir/quick_vmpool.out";
& vmpool -listall | Out-FileUtf8NoBom "$tempDir/quick_vmpool.out";

#
# quick_vmoprcmd.out
#
"Running command: vmoprcmd -d > $tempDir/quick_vmoprcmd.out";
& vmoprcmd -d | Out-FileUtf8NoBom "$tempDir/quick_vmoprcmd.out";

#
# quick_vmoprcmd_devconfig.out
#
"Running command: vmoprcmd -devconfig -l > $tempDir/quick_vmoprcmd_devconfig.out";
& vmoprcmd -devconfig -l | Out-FileUtf8NoBom "$tempDir/quick_vmoprcmd_devconfig.out";

#
# quick_nbdevquery_listdp.out
#
"Running command: nbdevquery -listdp -U > $tempDir/quick_nbdevquery_listdp.out";
& nbdevquery -listdp -U | Out-FileUtf8NoBom "$tempDir/quick_nbdevquery_listdp.out";

#
# quick_nbdevquery_liststs.out
#
"Running command: nbdevquery -liststs -U > $tempDir/quick_nbdevquery_liststs.out";
& nbdevquery -liststs -U | Out-FileUtf8NoBom "$tempDir/quick_nbdevquery_liststs.out";

#
# quick_nbdevquery_listdv_pt.out
#
"Running command: nbdevquery -listdv -stype IBM-ProtectTier -U > $tempDir/quick_nbdevquery_listdv_pt.out";
& nbdevquery -listdv -stype IBM-ProtectTier -U | Out-FileUtf8NoBom "$tempDir/quick_nbdevquery_listdv_pt.out";

#
# quick_nbdevquery_listdv_pt_d.out
#
"Running command: nbdevquery -listdv -stype IBM-ProtectTier -D > $tempDir/quick_nbdevquery_listdv_pt_d.out";
& nbdevquery -listdv -stype IBM-ProtectTier -D | Out-FileUtf8NoBom "$tempDir/quick_nbdevquery_listdv_pt_d.out";

#
# $tempDir/quick_nbdevquery_listdv_dd.out
#
"Running command: nbdevquery -listdv -stype DataDomain -U > $tempDir/quick_nbdevquery_listdv_dd.out";
& nbdevquery -listdv -stype DataDomain -U | Out-FileUtf8NoBom "$tempDir/quick_nbdevquery_listdv_dd.out";

#
# quick_nbdevquery_listdv_dd_d.out
#
"Running command: nbdevquery -listdv -stype DataDomain -D > $tempDir/quick_nbdevquery_listdv_dd_d.out";
& nbdevquery -listdv -stype DataDomain -D | Out-FileUtf8NoBom "$tempDir/quick_nbdevquery_listdv_dd_d.out";

#
# quick_nbdevquery_listdv_pd.out
#
"Running command: nbdevquery -listdv -stype PureDisk -U > $tempDir/quick_nbdevquery_listdv_pd.out"
& nbdevquery -listdv -stype PureDisk -U | Out-FileUtf8NoBom "$tempDir/quick_nbdevquery_listdv_pd.out";

#
# quick_nbdevquery_listdv_pd_d.out
#
"Running command: nbdevquery -listdv -stype PureDisk -D > $tempDir/quick_nbdevquery_listdv_pd_d.out"
& nbdevquery -listdv -stype PureDisk -D | Out-FileUtf8NoBom "$tempDir/quick_nbdevquery_listdv_pd_d.out";

#
# quick_nbdevquery_listdv_ad.out
#
"Running command: nbdevquery -listdv -stype AdvancedDisk -U > $tempDir/quick_nbdevquery_listdv_ad.out"
& nbdevquery -listdv -stype AdvancedDisk -U | Out-FileUtf8NoBom "$tempDir/quick_nbdevquery_listdv_ad.out";

#
# quick_nbdevquery_listdv_ad_d.out
#
"Running command: nbdevquery -listdv -stype AdvancedDisk -D > $tempDir/quick_nbdevquery_listdv_ad_d.out"
& nbdevquery -listdv -stype AdvancedDisk -D | Out-FileUtf8NoBom "$tempDir/quick_nbdevquery_listdv_ad_d.out";

#
# quick_nbdevquery_listglobals.out
#
"Running command: nbdevquery -listglobals -U > $tempDir/quick_nbdevquery_listglobals.out"
& nbdevquery -listglobals -U | Out-FileUtf8NoBom "$tempDir/quick_nbdevquery_listglobals.out";

#
# quick_bpdbjobssummary.out
#
"Running command: bpdbjobs -summary > $tempDir/quick_bpdbjobssummary.out"
& bpdbjobs -summary | Out-FileUtf8NoBom "$tempDir/quick_bpdbjobssummary.out";

#
# quick_bpdbjobs.out
#
"Running command: bpdbjobs -report -all_columns > $tempDir/quick_bpdbjobs.out"
& bpdbjobs -report -all_columns | Out-FileUtf8NoBom "$tempDir/quick_bpdbjobs.out";

#
# quick_nbftconfig_listclients.out
#
"Running command: nbftconfig -listclients -verbose > $tempDir/quick_nbftconfig_listclients.out"
& nbftconfig -listclients -verbose | Out-FileUtf8NoBom "$tempDir/quick_nbftconfig_listclients.out";

#
# quick_nbftconfig_listservers.out
#
"Running command: nbftconfig -listservers -verbose > $tempDir/quick_nbftconfig_listservers.out"
& nbftconfig -listservers -verbose | Out-FileUtf8NoBom "$tempDir/quick_nbftconfig_listservers.out";

#
# quick_version.out
#
Get-Content "$env:ProgramFiles\veritas\netbackup\version.txt" | Out-FileUtf8NoBom "$tempDir/quick_version.out";


#
# Time information
#
$startTime=[System.DateTime]::Now;
$startTime.ToString("hh tt", [System.Globalization.CultureInfo]::InvariantCulture.DateTimeFormat) > "$tempDir/quick_time2.out";

#
# Completed with the standard scan.
#
#$title = "Optional Client Version and Policy Coverage collection";
#$message = "Do you want to collect data on client versions and policy coverage?";
#$yes = New-Object System.Management.Automation.Host.ChoiceDescription "&Yes", "Runs the additional collection.";
#$no = New-Object System.Management.Automation.Host.ChoiceDescription "&No", "Skips that optional step and finishes the collection.";
#$options = [System.Management.Automation.Host.ChoiceDescription[]]($yes, $no)
#
#$result = $host.ui.PromptForChoice($title, $message, $options, 0) 
#
#if ($result -eq 0) {
#
#    Get-Content "$tempDir/quick_bpplclients.out" | Select-Object -Skip 2 | ForEach-Object { 
#
#        $coverage=($_ -split " ")[2];
#        
#        "Running command: bpcoverage -c $coverage >> $tempDir/quick_bpcoverage.out";
#        bpcoverage -c $coverage >> "$tempDir/quick_bpcoverage.out";
#    }
#}

#
# Clean up...
#
"File collection is complete."
"Files are located in the following folder:"

(Get-Item $tempDir).FullName;

"Launch the Live Optics collector and add this folder to your NetBackup collection.";
# SIG # Begin signature block
# MIIb+AYJKoZIhvcNAQcCoIIb6TCCG+UCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCC5ne4+IxUEuRdX
# 2s6OshoAKDvdv1vzcIm/+w+vgtE9mqCCCh0wggT4MIID4KADAgECAhBzZpXvvr/L
# YwAAAABVZluYMA0GCSqGSIb3DQEBCwUAMIG0MQswCQYDVQQGEwJVUzEWMBQGA1UE
# ChMNRW50cnVzdCwgSW5jLjEoMCYGA1UECxMfU2VlIHd3dy5lbnRydXN0Lm5ldC9s
# ZWdhbC10ZXJtczE5MDcGA1UECxMwKGMpIDIwMTUgRW50cnVzdCwgSW5jLiAtIGZv
# ciBhdXRob3JpemVkIHVzZSBvbmx5MSgwJgYDVQQDEx9FbnRydXN0IENvZGUgU2ln
# bmluZyBDQSAtIE9WQ1MxMB4XDTE4MDMxNjE3NTgyMloXDTE5MDMxNjE4MjgxM1ow
# WDELMAkGA1UEBhMCVVMxDjAMBgNVBAgTBVRleGFzMRMwEQYDVQQHEwpSb3VuZCBS
# b2NrMREwDwYDVQQKEwhEZWxsIEluYzERMA8GA1UEAxMIRGVsbCBJbmMwggEiMA0G
# CSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQDV0MXSafqENSvohK+axYm2szyWUneb
# qjwuSZyrrR5IK7JrbjXShlM4H23eyalb/urqSkFQ+Ox21Fkkb28aYARakg2a4W5h
# ix5DjOBGyQ0A4hIRiSa9YC0Pl/nMqtmorGv1PYcy5a4MatQkwdi7XMgorvoJYobR
# 99Qs+TENQVyz85UHKhliaEHwk6aPvrNqxKaqANe4PqZEB24ogjDK09XtOQ2R5OXs
# eHk0gB+TqmnkwzK5DxTSwpiuLIolC61l7b+oxgkawa+/6g+O8sLtzUmedqiQUdtt
# qekivkHjAUWtOZ5Ve5bDx5cH7OcAODoLGsbzmX9sTjlqs5QIWMRyOfM1AgMBAAGj
# ggFfMIIBWzAOBgNVHQ8BAf8EBAMCB4AwEwYDVR0lBAwwCgYIKwYBBQUHAwMwagYI
# KwYBBQUHAQEEXjBcMCMGCCsGAQUFBzABhhdodHRwOi8vb2NzcC5lbnRydXN0Lm5l
# dDA1BggrBgEFBQcwAoYpaHR0cDovL2FpYS5lbnRydXN0Lm5ldC9vdmNzMS1jaGFp
# bjI1Ni5jZXIwMQYDVR0fBCowKDAmoCSgIoYgaHR0cDovL2NybC5lbnRydXN0Lm5l
# dC9vdmNzMS5jcmwwSgYDVR0gBEMwQTA2BgpghkgBhvpsCgEDMCgwJgYIKwYBBQUH
# AgEWGmh0dHA6Ly93d3cuZW50cnVzdC5uZXQvcnBhMAcGBWeBDAEEMB8GA1UdIwQY
# MBaAFH4aHxoRdFxkyQwflAGr/YFkLqEsMB0GA1UdDgQWBBT5Z797PfJzX8seCrnX
# L3gauHsnHTAJBgNVHRMEAjAAMA0GCSqGSIb3DQEBCwUAA4IBAQCkZP/Z2dTU30+Y
# fEs5tstA95kLGXwZuhSjVH7W2/Y5rzyFfaRjmHQBTIie74XYP8yax/QIj4oF4S/h
# Z0ufnx33dmNrT0I4olVd6IAME4PqeDUtAxc8AXIqCayJa1NLeSJOWZPMyNoFpb+Q
# Jj4hQ0VJel8wXQZ1oAqxch/zrLEdtb9Zg+iQRa6lKEPDZoyeN4WdNQdam76YBpht
# TTjMKgKGkyo2HXlC5b9e7BJ0b/j97O+xH3TrWioYCA9rh7tsHptSulg7ooEYyMA6
# n2MrCdOENLcBK6hxmdTiQFQ3fkqr12TNoqLrRUAEuzBpvXF/KycAq0AupNZR/FZ/
# TFfvJjZsMIIFHTCCBAWgAwIBAgIMQ8ELHAAAAABR03PaMA0GCSqGSIb3DQEBCwUA
# MIG+MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEoMCYGA1UE
# CxMfU2VlIHd3dy5lbnRydXN0Lm5ldC9sZWdhbC10ZXJtczE5MDcGA1UECxMwKGMp
# IDIwMDkgRW50cnVzdCwgSW5jLiAtIGZvciBhdXRob3JpemVkIHVzZSBvbmx5MTIw
# MAYDVQQDEylFbnRydXN0IFJvb3QgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkgLSBH
# MjAeFw0xNTA2MTAxMzQ2MDVaFw0zMDExMTAxNDE2MDVaMIG0MQswCQYDVQQGEwJV
# UzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEoMCYGA1UECxMfU2VlIHd3dy5lbnRy
# dXN0Lm5ldC9sZWdhbC10ZXJtczE5MDcGA1UECxMwKGMpIDIwMTUgRW50cnVzdCwg
# SW5jLiAtIGZvciBhdXRob3JpemVkIHVzZSBvbmx5MSgwJgYDVQQDEx9FbnRydXN0
# IENvZGUgU2lnbmluZyBDQSAtIE9WQ1MxMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8A
# MIIBCgKCAQEA2IYNoGQKavWOlro7mmhq20RA3vUt/OttRFjPjnLDcC1eWtT5C9gR
# NZfAKfE77heh1kS5vx9KmOAZDrXjWQgdUoygsIooQaShhkBQs03HiEHaNpOnTD1j
# LAPLr40AJRFpUCL2SwDOw7IugYXdI91OTbXwwKh0/OtsYCmYcY9o97cd8/hDi2KV
# MoZPyshMS1P+USAwHv8ARMdecZqlG4l0Sv5TPoTqsaOJnfgtQljYCgZQI3AWHAJ3
# lxg82RB9DHrmBSwaCBZmEjhscR3WZNdWwI02w7i1ALjdL7V6v7Gkbnj6bRJ25Jl3
# EaYk5IK5RnmpsKNJW9NTdEOBdG+h+C4vJQIDAQABo4IBITCCAR0wDgYDVR0PAQH/
# BAQDAgEGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMBIGA1UdEwEB/wQIMAYBAf8CAQAw
# MwYIKwYBBQUHAQEEJzAlMCMGCCsGAQUFBzABhhdodHRwOi8vb2NzcC5lbnRydXN0
# Lm5ldDAwBgNVHR8EKTAnMCWgI6Ahhh9odHRwOi8vY3JsLmVudHJ1c3QubmV0L2cy
# Y2EuY3JsMDsGA1UdIAQ0MDIwMAYEVR0gADAoMCYGCCsGAQUFBwIBFhpodHRwOi8v
# d3d3LmVudHJ1c3QubmV0L3JwYTAdBgNVHQ4EFgQUfhofGhF0XGTJDB+UAav9gWQu
# oSwwHwYDVR0jBBgwFoAUanImetAe733nO2lR1GyNn5ASZqswDQYJKoZIhvcNAQEL
# BQADggEBALd0Z7Q+TCUvYjWBnNJ25afiKHh0w+wZ5RP5tiOUT2KnTOZOcsIkkNB5
# uqjN10TfADkw8SjEcm94rJALLaPqha0UwgsUmT/7I3StqKrMImMyJQErytAWRkJa
# p0RHWB0EQeLRMb6XYTuuBBhsenqBX6si/gjWNY6nDfXsINzcbceidtj7ZqhDAwEh
# qKD2TObUgs7XDQHWM6yQo+CSuxgCdUMCegS02BXtV5E8EueNZvi9FMxGgtb6A8AK
# DXyxgA/roRo2pXUfPNg66ueVgd6sWvudtcwRTip0EdZ/eUJhyiFBYB6k3fHZdLDV
# pegLTIGJ38yGFiUZpRRXlaKKInuxZY0xghExMIIRLQIBATCByTCBtDELMAkGA1UE
# BhMCVVMxFjAUBgNVBAoTDUVudHJ1c3QsIEluYy4xKDAmBgNVBAsTH1NlZSB3d3cu
# ZW50cnVzdC5uZXQvbGVnYWwtdGVybXMxOTA3BgNVBAsTMChjKSAyMDE1IEVudHJ1
# c3QsIEluYy4gLSBmb3IgYXV0aG9yaXplZCB1c2Ugb25seTEoMCYGA1UEAxMfRW50
# cnVzdCBDb2RlIFNpZ25pbmcgQ0EgLSBPVkNTMQIQc2aV776/y2MAAAAAVWZbmDAN
# BglghkgBZQMEAgEFAKCBpjAZBgkqhkiG9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgor
# BgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIBFTAvBgkqhkiG9w0BCQQxIgQggvwhqmAv
# M4lGjCBgRceDgcFkZpJQcHkr274EJXViejQwOgYKKwYBBAGCNwIBDDEsMCqgKIAm
# AEQAZQBsAGwAIABJAG4AYwAuACAATgBlAHQAYgBhAGMAawB1AHAwDQYJKoZIhvcN
# AQEBBQAEggEAJNUdrCgAWSULpOnXvJNil15Gv7i4wPw/icOCb9rBBwKClIkOpyu7
# UzOcZqLC3abQidfhs9Pz5XJPyHUOYFfUhfVwVsm7MSwszhfXoDpBsA9WTwvM+TdF
# JgeG/PmRvFxAa7TYG9rAZduWGKrDD+rrB1Mshe064PZlFiYEachPGlzwVmh6Tger
# pqf2gq3GkFTc0Y7cuLTZ/YUnTpH3svQ8Vpqvn+k1gDqhfahMpuf5RvDsp8ptSoul
# MlpCRwPOB7ML71lySILQueDHDirwjyoe5di0oQQUNtrYKis/WAsByC+2TVUyi916
# 9lUM5njqy3Kzg2kGHczGrHHNru+jkCVkuqGCDo8wgg6LBgorBgEEAYI3AwMBMYIO
# ezCCDncGCSqGSIb3DQEHAqCCDmgwgg5kAgEDMQ8wDQYJYIZIAWUDBAIBBQAwgfIG
# CyqGSIb3DQEJEAEEoIHiBIHfMIHcAgEBBglghkiG+mwKAwUwMTANBglghkgBZQME
# AgEFAAQgmPmbom7ElGtN0YYzNc3VhGwiKH0ek/Wwke35E6fK/7sCBlsRdW7eJxgT
# MjAxODExMTQxNjE4NTQuNTAzWjAEgAIB9KB2pHQwcjELMAkGA1UEBhMCQ0ExEDAO
# BgNVBAgTB09udGFyaW8xDzANBgNVBAcTBk90dGF3YTEWMBQGA1UEChMNRW50cnVz
# dCwgSW5jLjEoMCYGA1UEAxMfRW50cnVzdCBUaW1lIFN0YW1waW5nIEF1dGhvcml0
# eaCCCe8wggTUMIIDvKADAgECAhEAzXDmaahZIxEAAAAAVZHa3TANBgkqhkiG9w0B
# AQsFADCBsjELMAkGA1UEBhMCVVMxFjAUBgNVBAoTDUVudHJ1c3QsIEluYy4xKDAm
# BgNVBAsTH1NlZSB3d3cuZW50cnVzdC5uZXQvbGVnYWwtdGVybXMxOTA3BgNVBAsT
# MChjKSAyMDE1IEVudHJ1c3QsIEluYy4gLSBmb3IgYXV0aG9yaXplZCB1c2Ugb25s
# eTEmMCQGA1UEAxMdRW50cnVzdCBUaW1lc3RhbXBpbmcgQ0EgLSBUUzEwHhcNMTgw
# NjAxMTU0MzQ3WhcNMjkwOTAxMTYxMzQ3WjByMQswCQYDVQQGEwJDQTEQMA4GA1UE
# CBMHT250YXJpbzEPMA0GA1UEBxMGT3R0YXdhMRYwFAYDVQQKEw1FbnRydXN0LCBJ
# bmMuMSgwJgYDVQQDEx9FbnRydXN0IFRpbWUgU3RhbXBpbmcgQXV0aG9yaXR5MIIB
# IjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAqkcURW9tTf+GlbffWIQd22el
# +8xm29vQQirN/d0uIZh6odz22IDJyKmxMSBHM4PAx+BXxFNnEsoaiSNoaugtvMT9
# fRNx38ssWkBoSx+kkFbaFQCSOM4KdVbvkaHXN7ny60HBr4C4A0PahR6MfQyRFgey
# HY5shRysKcSffA/m5FT1NfAIZr8DGmUDzXQNnwJCGVlxTt1e94evLxZECiq5/JEi
# NZEtzsb4DJTgaemnc2Tr6m8bSNSXIQtyYW4DLoW1k14if/neq1+Sre4xzbRll+FN
# 5f8eIRcZG3o9hiO0WlnVw1L04lnKZ/KvcKA83YiiWzr7awPOXvC6OSucF0oDuwID
# AQABo4IBIjCCAR4wDgYDVR0PAQH/BAQDAgeAMBYGA1UdJQEB/wQMMAoGCCsGAQUF
# BwMIMEEGA1UdIAQ6MDgwNgYKYIZIAYb6bAoDBTAoMCYGCCsGAQUFBwIBFhpodHRw
# Oi8vd3d3LmVudHJ1c3QubmV0L3JwYTAJBgNVHRMEAjAAMDMGCCsGAQUFBwEBBCcw
# JTAjBggrBgEFBQcwAYYXaHR0cDovL29jc3AuZW50cnVzdC5uZXQwMQYDVR0fBCow
# KDAmoCSgIoYgaHR0cDovL2NybC5lbnRydXN0Lm5ldC90czFjYS5jcmwwHwYDVR0j
# BBgwFoAUw8Jx0nvXaAWuOzmbNCUMYgPHV2gwHQYDVR0OBBYEFBogt9wcagoW7dMY
# 5UwcJOPrJsrLMA0GCSqGSIb3DQEBCwUAA4IBAQCxAdm3XIWVt84iEurQMcjAe2cq
# rmrH2wGNLcUXLeM57WT22xB/SqAIx9apH1oON530lzlew8TpzTsW7h5SqHqJ+R2t
# 6pXroZEO997UaRDg3iG3z8EHmDFaoudzPsq+znUQBVTgcWPraOLKdhJIr3eTZ5Go
# cZJ1I8uVPcMevUBi3JYQuxNgroUCk2tYFrRbA5Ifxde9fSKIxhTqBahMsdjkLfhj
# 1wi6VFL6ab2U4F3z3HnvcVZ7kB/VoTfzadEZpeQHSOIJmOdN78m5kQwm77QHZtvB
# leLgkMmMybN2LWom/KXk0AWnPcP0+hx8ktF2tlk0ySLuc2qyXK7PVyoY+CfBMIIF
# EzCCA/ugAwIBAgIMWNoT/wAAAABRzg33MA0GCSqGSIb3DQEBCwUAMIG0MRQwEgYD
# VQQKEwtFbnRydXN0Lm5ldDFAMD4GA1UECxQ3d3d3LmVudHJ1c3QubmV0L0NQU18y
# MDQ4IGluY29ycC4gYnkgcmVmLiAobGltaXRzIGxpYWIuKTElMCMGA1UECxMcKGMp
# IDE5OTkgRW50cnVzdC5uZXQgTGltaXRlZDEzMDEGA1UEAxMqRW50cnVzdC5uZXQg
# Q2VydGlmaWNhdGlvbiBBdXRob3JpdHkgKDIwNDgpMB4XDTE1MDcyMjE5MDI1NFoX
# DTI5MDYyMjE5MzI1NFowgbIxCzAJBgNVBAYTAlVTMRYwFAYDVQQKEw1FbnRydXN0
# LCBJbmMuMSgwJgYDVQQLEx9TZWUgd3d3LmVudHJ1c3QubmV0L2xlZ2FsLXRlcm1z
# MTkwNwYDVQQLEzAoYykgMjAxNSBFbnRydXN0LCBJbmMuIC0gZm9yIGF1dGhvcml6
# ZWQgdXNlIG9ubHkxJjAkBgNVBAMTHUVudHJ1c3QgVGltZXN0YW1waW5nIENBIC0g
# VFMxMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA2SPmFKTofEuFcVj7
# +IHmcotdRsOIAB840Irh1m5WMOWv2mRQfcITOfu9ZrTahPuD0Cgfy3boYFBpm/PO
# TxPiwT7B3xLLMqP4XkQiDsw66Y1JuWB0yN5UPUFeQ18oRqmmt8oQKyK8W01bjBdl
# Eob9LHfVxaCMysKD4EdXfOdwrmJFJzEYCtTApBhVUvdgxgRLs91oMm4QHzQRuBJ4
# ZPHuqeD347EijzRaZcuK9OFFUHTfk5emNObQTDufN0lSp1NOny5nXO2W/KW/dFGI
# 46qOvdmxL19QMBb0UWAia5nL/+FUO7n7RDilCDkjm2lH+jzE0Oeq30ay7PKKGawp
# sjiVdQIDAQABo4IBIzCCAR8wEgYDVR0TAQH/BAgwBgEB/wIBADAOBgNVHQ8BAf8E
# BAMCAQYwOwYDVR0gBDQwMjAwBgRVHSAAMCgwJgYIKwYBBQUHAgEWGmh0dHA6Ly93
# d3cuZW50cnVzdC5uZXQvcnBhMDMGCCsGAQUFBwEBBCcwJTAjBggrBgEFBQcwAYYX
# aHR0cDovL29jc3AuZW50cnVzdC5uZXQwMgYDVR0fBCswKTAnoCWgI4YhaHR0cDov
# L2NybC5lbnRydXN0Lm5ldC8yMDQ4Y2EuY3JsMBMGA1UdJQQMMAoGCCsGAQUFBwMI
# MB0GA1UdDgQWBBTDwnHSe9doBa47OZs0JQxiA8dXaDAfBgNVHSMEGDAWgBRV5IHR
# EYC+2Im5CKMx+aEkCRa5cDANBgkqhkiG9w0BAQsFAAOCAQEAHSTnmnRbqnD8sQ4x
# RdcsAH9mOiugmjSqrGNtifmf3w13/SQj/E+ct2+P8/QftsH91hzEjIhmwWONuld3
# 07gaHshRrcxgNhqHaijqEWXezDwsjHS36FBD08wo6BVsESqfFJUpyQVXtWc26Dyp
# g+9BwSEW0373LRFHZnZgghJpjHZVcw/fL0td6Wwj+Af2tX3WaUWcWH1hLvx4S0NO
# iZFGRCygU6hFofYWWLuRE/JLxd8LwOeuKXq9RbPncDDnNI7revbTtdHeaxOZRrOL
# 0k2TdbXxb7/cACjCJb+856NlNOw/DR2XjPqqiCKkGDXbBY524xDIKY9j0K6sGNna
# xJ9REjGCA2QwggNgAgEBMIHIMIGyMQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50
# cnVzdCwgSW5jLjEoMCYGA1UECxMfU2VlIHd3dy5lbnRydXN0Lm5ldC9sZWdhbC10
# ZXJtczE5MDcGA1UECxMwKGMpIDIwMTUgRW50cnVzdCwgSW5jLiAtIGZvciBhdXRo
# b3JpemVkIHVzZSBvbmx5MSYwJAYDVQQDEx1FbnRydXN0IFRpbWVzdGFtcGluZyBD
# QSAtIFRTMQIRAM1w5mmoWSMRAAAAAFWR2t0wDQYJYIZIAWUDBAIBBQCgggFsMBoG
# CSqGSIb3DQEJAzENBgsqhkiG9w0BCRABBDAvBgkqhkiG9w0BCQQxIgQgMomtn5oT
# JIyn4y1Mb1Sq+X8V8QsBRo5tKZ21vm7Da3EwggEbBgsqhkiG9w0BCRACDDGCAQow
# ggEGMIIBAjCB5wQUSdhZxXIaXt9CzQql2YpizJnknNkwgc4wgbikgbUwgbIxCzAJ
# BgNVBAYTAlVTMRYwFAYDVQQKEw1FbnRydXN0LCBJbmMuMSgwJgYDVQQLEx9TZWUg
# d3d3LmVudHJ1c3QubmV0L2xlZ2FsLXRlcm1zMTkwNwYDVQQLEzAoYykgMjAxNSBF
# bnRydXN0LCBJbmMuIC0gZm9yIGF1dGhvcml6ZWQgdXNlIG9ubHkxJjAkBgNVBAMT
# HUVudHJ1c3QgVGltZXN0YW1waW5nIENBIC0gVFMxAhEAzXDmaahZIxEAAAAAVZHa
# 3TAWBBTb6daIuK+CUjlunPOQQbu6MvbVFTANBgkqhkiG9w0BAQsFAASCAQALCFe3
# 19hSbq8u1QCki1YvH/fCV1swIsKJC5nsGjaP1q1+UZkciScFxrVVgjhzYdssM41R
# J3ThpvwEZy6uox3owsZ5tq6bkmbzHGT4SiWE0GiXANppY+O6Qzc56n/ZYA7BiJ1k
# Zx2quoHbxEfTqIA5riUNMOEHnV2WSPohW+VOSCa2DrqfmkK0PVEAP3sUy68zceQS
# NNHJTyXccm1nmnw+INOm1b5cXpLYk8mMk1TD0LpaN9EDmSxb5gbU0g2+3H2oYqiH
# WC0JaMJ/IpDXj++OF+WkLfVecZJ1Xh+YcUWu/VxetlGnOxWSyYYQ06cyKRech+IA
# R3CnPTvf1UHV7uIj
# SIG # End signature block
