﻿<#

.SYNOPSIS
This Powershell script can be run on a server to gather a file collection from a NetWorker instance.
The generated files can be uploaded to LiveOpics through the collector for analysis. 

.DESCRIPTION
This script will run queries against NetWorker and produce .out files for analysis.

.LINK
https://www.liveoptics.com/

#>

#-----------------------------------------------------------------------------
#  Copyright (c) 2017-2018 by Dell Inc. 
# 
#  All rights reserved.  
# 
#-----------------------------------------------------------------------------

$scriptVersion="1.0.013";

""
"------------------------------------------------------";
"Live Optics collection script for NetWorker on Windows";
"------------------------------------------------------";
$date=Get-Date -Format o;
"Date: " + $date;
"Version: " + $scriptVersion;
""

#
# UTF-8 output to file without BOM
#
function Out-FileUtf8NoBom {

  [CmdletBinding()]
  param(
    [Parameter(Mandatory=$True, Position=0)] [string] $LiteralPath,
    [switch] $Append,
    [switch] $NoClobber,
    [Parameter(ValueFromPipeline=$True)]
    [string[]]$lines
  )

  Begin
  {
	# Make sure that the .NET framework sees the same working dir. as PS
	# and resolve the input path to a full path.
	[System.IO.Directory]::SetCurrentDirectory($PWD) # Caveat: .NET Core doesn't support [Environment]::CurrentDirectory
	$LiteralPath = [IO.Path]::GetFullPath($LiteralPath)

	# If -NoClobber was specified, throw an exception if the target file already
	# exists.
	if ($NoClobber -and (Test-Path $LiteralPath)) {
		Throw [IO.IOException] "The file '$LiteralPath' already exists."
	}

	try {
		
		# Create a StreamWriter object.
		# Note that we take advantage of the fact that the StreamWriter class by default:
		# - uses UTF-8 encoding
		# - without a BOM.
		$sw = New-Object IO.StreamWriter $LiteralPath, $Append

	}
	catch {
		Write-Error "Failed to create output stream."
		Write-Error $_;
		exit;
    }	
  }
  
  Process { 
	try {
		ForEach ($line in $lines) {
			$sw.WriteLine($line)
		}
	}
	catch {
		$sw.Dispose();
		$sw = $null;
		
		Write-Error "Failed to write to stream."
		Write-Error $_;
		exit;
	}
  }
  
  End {
	if ($sw) {
		$sw.Dispose();
	}
  }
}

#
# Create a temporary directory
#
$dt=Get-Date -UFormat %Y%m%d%H%M%S
$tempDir="LiveOpticsNetWorker" + $dt
new-item -type directory -path $tempDir | Out-Null;

if (!(Test-Path $tempDir)) {
    
    Write-Error "Failed to create directory to hold file output: $tempDir";
    exit;
}

"Temporary directory created: " + $tempDir;

#
# Set the path
#
$env:Path="$env:Path;$env:ProgramFiles/Legato/nsr/bin";

#
# Date information - quick_date.out
#
$quick_date="$tempDir/nwquick_date.out";
$date=Get-Date -Uformat "%a %m/%d/%Y"
$date | Out-FileUtf8NoBom $quick_date;

#
# Improved date format
#
$quick_psdate="$tempDir/quick_psdate.out";
$date=Get-Date -Format o;
$date | Out-FileUtf8NoBom $quick_psdate;

#
# Script version
#
$quick_scriptVersion="$tempDir/nwquick_scriptVersion.out";
$scriptVersion | Out-FileUtf8NoBom $quick_scriptVersion;

#
#Windows version
#
$quick_osVersion="$tempDir/nwquick_windowsversion.out";
[environment]::OSVersion.Version | Out-FileUtf8NoBom $quick_osVersion

#
#Powershell version
#
$quick_psVersion="$tempDir/nwquick_psversion.out";
$PSVersionTable.PSVersion | Out-FileUtf8NoBom $quick_psVersion

#
# Time information
#
$startTime=[System.DateTime]::Now;
$startTime.ToString("hh tt", [System.Globalization.CultureInfo]::InvariantCulture.DateTimeFormat) | Out-FileUtf8NoBom "$tempDir/nwquick_time.out";

#
# Timezone information - quick_timezone.out
#
$tz=(& reg query "HKLM\SYSTEM\CurrentControlSet\Control\TimeZoneInformation");
$quick_timezone="$tempDir/nwquick_timezone.out";
$tz | Out-FileUtf8NoBom $quick_timezone;

#
# Hostname information - quick_hostname.out
#
$hostnameFile="$tempDir/nwquick_hostname.out";
hostname | Out-FileUtf8NoBom $hostnameFile;

#
# Base date used for drawing the line on backups.
# 120 days is the default.
#
$last=(New-TimeSpan -Days (120));
$lastBackupDate=[System.DateTime]::UtcNow.Subtract($last);
$lastBackupDateString="-t " + $lastBackupDate.ToString("MM\/dd\/yyyy");
$lastBackupDateString | Out-FileUtf8NoBom "$tempDir/nwquick_timeconstraint.out";

#
# Dump environment variables
#
foreach ($e in (gci env:*)) { $e.Name + "=" + $e.Value | Out-FileUtf8NoBom "$tempDir/nwquick_set.out" -Append; }

#
# NetWorker Commands
#
$nsradmin="nsradmin";
$nsrlic="nsrlic";
$mminfo="mminfo";

$commandFile="$tempDir/CommandFile.txt";

"show" | Out-FileUtf8NoBom $commandFile;
"option hidden" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append; 

"Running command: nsradmin -i";

& $nsradmin -i "$commandFile" 2> "$tempDir/nwquick_nsradmin_err.out" | Out-FileUtf8NoBom "$tempDir/nwquick_nsradmin.out";

#
# Check for errors
#
$errorFile="$tempDir/nwquick_nsradmin_err.out";

if ((test-path $errorFile) -and ((get-childitem $errorFile).Length -gt 0)) {
    Write-Error "ERROR.  Unable to run nsradmin.";
    exit;
}

"Running command: nsrlic -v";
& $nsrlic -v 2> $tempDir/nwquick_nsrlic_2_err.out | Out-FileUtf8NoBom $tempdir/nwquick_nsrlic.out;

$errorFile="$tempDir/nwquick_mminfojobs_err.out";

$errorString="unknown report constraint:";
$mminfojobsout="$tempDir/nwquick_mminfojobs.out";

"Running command: mminfo";
& $mminfo -av -xm $lastBackupDateString -r 'ssid,state,volume,client,pool,capacity,location,name,sscreate(50),sscomp(50),savetime(50),level,totalsize,nfiles,ssretent(50),copies,ssflags,group,ssbrowse(50),cloneid,clonetime(50),clflags,fragsize,sumsize,ssaccess(50),clretent(50),vmname,backup_size,attrs' 2> $errorFile | Out-FileUtf8NoBom $mminfojobsout;

#
# Check for errors
#
if (((test-path $errorFile) -and ((get-childitem $errorFile).Length -gt 0)) -or ($(get-content -first 1 $mminfojobsout) -like "*unknown report constraint*")) {

	"Trying mminfo command for NetWorker version 7.5 or older...";
	remove-item $mminfojobsout;
    remove-item $errorFile;

    & $mminfo -av -xm $lastBackupDateString -r 'ssid,state,volume,client,pool,capacity,location,name,sscreate(50),sscomp(50),savetime(50),level,totalsize,nfiles,ssretent(50),copies,ssflags,group,ssbrowse(50),cloneid,clonetime(50),clflags,fragsize,sumsize,ssaccess(50),clretent(50),attrs' 2> $errorFile | Out-FileUtf8NoBom $mminfojobsout;
    
    if (((test-path $errorFile) -and ((get-childitem $errorFile).Length -gt 0)) -or ($(get-content -first 1 $mminfojobsout) -like "*unknown report constraint*")) {
    
		"Trying mminfo command for oldest supported NetWorker version ...";
	
        remove-item $errorFile;
		remove-item $mminfojobsout;
        
        & $mminfo -av -xm $lastBackupDateString -r 'ssid,state,volume,client,pool,capacity,location,name,sscreate(50),sscomp(50),savetime(50),level,totalsize,nfiles,ssretent(50),copies,ssflags,group,ssbrowse(50),cloneid,clonetime(50),clflags,fragsize,sumsize,ssaccess(50),attrs' 2> $errorFile | Out-FileUtf8NoBom $mminfojobsout;
        
        if (((test-path $errorFile) -and ((get-childitem $errorFile).Length -gt 0)) -or ($(get-content -first 1 $mminfojobsout) -like "*unknown report constraint*")) {

            Write-Error "ERROR.  Failed to get job info.";
            cat $errorFile;
			cat $mminfojobsout;
            exit;
        }
    }
}
 
"Running command: mminfo -av -xm $lastBackupDateString -r ssid,ssid(53)";
& $mminfo -av -xm $lastBackupDateString -r "ssid,ssid(53)" 2> "$tempDir/nwquick_mminfojobs_long_err.out" | Out-FileUtf8NoBom "$tempDir/nwquick_mminfojobs_long.out"; 

"Running command: mminfo -av -xm Getting media info...";
& $mminfo -av -xm -r 'state,volume,written,pool,%used,read,capacity,avail,volflags,type,family,barcode,volid,location,volaccess,volretent(50),olabel,labeled,recycled,volattrs' 2> "$tempDir/nwquick_mminfomedia_err.out" | Out-FileUtf8NoBom "$tempDir/nwquick_mminfomedia.out";

#
# Check for errors
#
$errorFile="$tempDir/nwquick_mminfomedia_err.out";

if ((test-path $errorFile) -and ((get-childitem $errorFile).Length -gt 0)) {
    
    Write-Error "ERROR.  Failed run the mminfo for the media information.";
    exit;
}

"Running command: mminfo -X -a -xm";
& $mminfo -X -a -xm 2> "$tempDir/nwquick_mminfosummary_err.out" | Out-FileUtf8NoBom "$tempDir/nwquick_mminfosummary.out";

"Running command:  nsradmin -i";
"show" | Out-FileUtf8NoBom $commandFile;
"option hidden" | Out-FileUtf8NoBom $commandFile -Append;
". NSR" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR action" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR auditlog" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR client" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append
". NSR client fix" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR clone" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR Data Domain" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR data protection policy" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR device" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR directive" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR group" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR label" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR license" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR lockbox" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR NAS discovery task" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR policy" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR pool" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR Report Home" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR schedule" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR Snapshot Policy" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR stage" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR Storage Node" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR task" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR usergroup" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
". NSR VBA Server" | Out-FileUtf8NoBom $commandFile -Append;
"p" | Out-FileUtf8NoBom $commandFile -Append;
& "$nsradmin" -i "$commandFile" | Out-FileUtf8NoBom "$tempDir/nwquick_nsradmin_single.out" 2> "$tempDir/nwquick_nsradmin_single_err.out";

#
# Time information
#
$startTime=[System.DateTime]::Now;
$startTime.ToString("hh tt", [System.Globalization.CultureInfo]::InvariantCulture.DateTimeFormat) | Out-FileUtf8NoBom "$tempDir/nwquick_time2.out";

#
# Clean up...
#
"File collection is complete."
"Files are located in the following folder:"

(Get-Item $tempDir).FullName;

"Launch the Live Optics collector and add this folder to your NetWorker collection.";
# SIG # Begin signature block
# MIIb+AYJKoZIhvcNAQcCoIIb6TCCG+UCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCBDouX9dbr2NudR
# caYC8auOpsukSvUS/w3i+lgRplGpCqCCCh0wggT4MIID4KADAgECAhBzZpXvvr/L
# YwAAAABVZluYMA0GCSqGSIb3DQEBCwUAMIG0MQswCQYDVQQGEwJVUzEWMBQGA1UE
# ChMNRW50cnVzdCwgSW5jLjEoMCYGA1UECxMfU2VlIHd3dy5lbnRydXN0Lm5ldC9s
# ZWdhbC10ZXJtczE5MDcGA1UECxMwKGMpIDIwMTUgRW50cnVzdCwgSW5jLiAtIGZv
# ciBhdXRob3JpemVkIHVzZSBvbmx5MSgwJgYDVQQDEx9FbnRydXN0IENvZGUgU2ln
# bmluZyBDQSAtIE9WQ1MxMB4XDTE4MDMxNjE3NTgyMloXDTE5MDMxNjE4MjgxM1ow
# WDELMAkGA1UEBhMCVVMxDjAMBgNVBAgTBVRleGFzMRMwEQYDVQQHEwpSb3VuZCBS
# b2NrMREwDwYDVQQKEwhEZWxsIEluYzERMA8GA1UEAxMIRGVsbCBJbmMwggEiMA0G
# CSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQDV0MXSafqENSvohK+axYm2szyWUneb
# qjwuSZyrrR5IK7JrbjXShlM4H23eyalb/urqSkFQ+Ox21Fkkb28aYARakg2a4W5h
# ix5DjOBGyQ0A4hIRiSa9YC0Pl/nMqtmorGv1PYcy5a4MatQkwdi7XMgorvoJYobR
# 99Qs+TENQVyz85UHKhliaEHwk6aPvrNqxKaqANe4PqZEB24ogjDK09XtOQ2R5OXs
# eHk0gB+TqmnkwzK5DxTSwpiuLIolC61l7b+oxgkawa+/6g+O8sLtzUmedqiQUdtt
# qekivkHjAUWtOZ5Ve5bDx5cH7OcAODoLGsbzmX9sTjlqs5QIWMRyOfM1AgMBAAGj
# ggFfMIIBWzAOBgNVHQ8BAf8EBAMCB4AwEwYDVR0lBAwwCgYIKwYBBQUHAwMwagYI
# KwYBBQUHAQEEXjBcMCMGCCsGAQUFBzABhhdodHRwOi8vb2NzcC5lbnRydXN0Lm5l
# dDA1BggrBgEFBQcwAoYpaHR0cDovL2FpYS5lbnRydXN0Lm5ldC9vdmNzMS1jaGFp
# bjI1Ni5jZXIwMQYDVR0fBCowKDAmoCSgIoYgaHR0cDovL2NybC5lbnRydXN0Lm5l
# dC9vdmNzMS5jcmwwSgYDVR0gBEMwQTA2BgpghkgBhvpsCgEDMCgwJgYIKwYBBQUH
# AgEWGmh0dHA6Ly93d3cuZW50cnVzdC5uZXQvcnBhMAcGBWeBDAEEMB8GA1UdIwQY
# MBaAFH4aHxoRdFxkyQwflAGr/YFkLqEsMB0GA1UdDgQWBBT5Z797PfJzX8seCrnX
# L3gauHsnHTAJBgNVHRMEAjAAMA0GCSqGSIb3DQEBCwUAA4IBAQCkZP/Z2dTU30+Y
# fEs5tstA95kLGXwZuhSjVH7W2/Y5rzyFfaRjmHQBTIie74XYP8yax/QIj4oF4S/h
# Z0ufnx33dmNrT0I4olVd6IAME4PqeDUtAxc8AXIqCayJa1NLeSJOWZPMyNoFpb+Q
# Jj4hQ0VJel8wXQZ1oAqxch/zrLEdtb9Zg+iQRa6lKEPDZoyeN4WdNQdam76YBpht
# TTjMKgKGkyo2HXlC5b9e7BJ0b/j97O+xH3TrWioYCA9rh7tsHptSulg7ooEYyMA6
# n2MrCdOENLcBK6hxmdTiQFQ3fkqr12TNoqLrRUAEuzBpvXF/KycAq0AupNZR/FZ/
# TFfvJjZsMIIFHTCCBAWgAwIBAgIMQ8ELHAAAAABR03PaMA0GCSqGSIb3DQEBCwUA
# MIG+MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEoMCYGA1UE
# CxMfU2VlIHd3dy5lbnRydXN0Lm5ldC9sZWdhbC10ZXJtczE5MDcGA1UECxMwKGMp
# IDIwMDkgRW50cnVzdCwgSW5jLiAtIGZvciBhdXRob3JpemVkIHVzZSBvbmx5MTIw
# MAYDVQQDEylFbnRydXN0IFJvb3QgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkgLSBH
# MjAeFw0xNTA2MTAxMzQ2MDVaFw0zMDExMTAxNDE2MDVaMIG0MQswCQYDVQQGEwJV
# UzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEoMCYGA1UECxMfU2VlIHd3dy5lbnRy
# dXN0Lm5ldC9sZWdhbC10ZXJtczE5MDcGA1UECxMwKGMpIDIwMTUgRW50cnVzdCwg
# SW5jLiAtIGZvciBhdXRob3JpemVkIHVzZSBvbmx5MSgwJgYDVQQDEx9FbnRydXN0
# IENvZGUgU2lnbmluZyBDQSAtIE9WQ1MxMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8A
# MIIBCgKCAQEA2IYNoGQKavWOlro7mmhq20RA3vUt/OttRFjPjnLDcC1eWtT5C9gR
# NZfAKfE77heh1kS5vx9KmOAZDrXjWQgdUoygsIooQaShhkBQs03HiEHaNpOnTD1j
# LAPLr40AJRFpUCL2SwDOw7IugYXdI91OTbXwwKh0/OtsYCmYcY9o97cd8/hDi2KV
# MoZPyshMS1P+USAwHv8ARMdecZqlG4l0Sv5TPoTqsaOJnfgtQljYCgZQI3AWHAJ3
# lxg82RB9DHrmBSwaCBZmEjhscR3WZNdWwI02w7i1ALjdL7V6v7Gkbnj6bRJ25Jl3
# EaYk5IK5RnmpsKNJW9NTdEOBdG+h+C4vJQIDAQABo4IBITCCAR0wDgYDVR0PAQH/
# BAQDAgEGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMBIGA1UdEwEB/wQIMAYBAf8CAQAw
# MwYIKwYBBQUHAQEEJzAlMCMGCCsGAQUFBzABhhdodHRwOi8vb2NzcC5lbnRydXN0
# Lm5ldDAwBgNVHR8EKTAnMCWgI6Ahhh9odHRwOi8vY3JsLmVudHJ1c3QubmV0L2cy
# Y2EuY3JsMDsGA1UdIAQ0MDIwMAYEVR0gADAoMCYGCCsGAQUFBwIBFhpodHRwOi8v
# d3d3LmVudHJ1c3QubmV0L3JwYTAdBgNVHQ4EFgQUfhofGhF0XGTJDB+UAav9gWQu
# oSwwHwYDVR0jBBgwFoAUanImetAe733nO2lR1GyNn5ASZqswDQYJKoZIhvcNAQEL
# BQADggEBALd0Z7Q+TCUvYjWBnNJ25afiKHh0w+wZ5RP5tiOUT2KnTOZOcsIkkNB5
# uqjN10TfADkw8SjEcm94rJALLaPqha0UwgsUmT/7I3StqKrMImMyJQErytAWRkJa
# p0RHWB0EQeLRMb6XYTuuBBhsenqBX6si/gjWNY6nDfXsINzcbceidtj7ZqhDAwEh
# qKD2TObUgs7XDQHWM6yQo+CSuxgCdUMCegS02BXtV5E8EueNZvi9FMxGgtb6A8AK
# DXyxgA/roRo2pXUfPNg66ueVgd6sWvudtcwRTip0EdZ/eUJhyiFBYB6k3fHZdLDV
# pegLTIGJ38yGFiUZpRRXlaKKInuxZY0xghExMIIRLQIBATCByTCBtDELMAkGA1UE
# BhMCVVMxFjAUBgNVBAoTDUVudHJ1c3QsIEluYy4xKDAmBgNVBAsTH1NlZSB3d3cu
# ZW50cnVzdC5uZXQvbGVnYWwtdGVybXMxOTA3BgNVBAsTMChjKSAyMDE1IEVudHJ1
# c3QsIEluYy4gLSBmb3IgYXV0aG9yaXplZCB1c2Ugb25seTEoMCYGA1UEAxMfRW50
# cnVzdCBDb2RlIFNpZ25pbmcgQ0EgLSBPVkNTMQIQc2aV776/y2MAAAAAVWZbmDAN
# BglghkgBZQMEAgEFAKCBpjAZBgkqhkiG9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgor
# BgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIBFTAvBgkqhkiG9w0BCQQxIgQgpy+/34/d
# VWNFzKTRKIov1bpX+Xj4CcuY1LJiIzd7OyAwOgYKKwYBBAGCNwIBDDEsMCqgKIAm
# AEQAZQBsAGwAIABJAG4AYwAuACAATgBlAHQAdwBvAHIAawBlAHIwDQYJKoZIhvcN
# AQEBBQAEggEAkj5FK9stjI7vsjxwgPnMgbg3UDYSu19fx3TtFtwf74uGqivQINqb
# nJc+HlszGNuoWXNtTRwuXZefXbY2xuwZqkYIsG/WnSNeM+PvXPgvQZS4fo/sT6yG
# JIWWoOvnj7fH2q62qcEePbZvNgseIMlcQDu0y3RoYgwFcqYCGRLZe6TeH5Wu3emw
# P4cWBDE5b9i1F7Tkuc3HPOTyaJ2u/Lv+8Dx6/2yDSt8Ea2RIfMdPxnSKQIUkQx6J
# 6Bl/PVN0J4B9xsxHB5cPkYIh5uTttbvWEdxSMX1ZkHb+YJ63UEd8jl0QvLcR2TtS
# 0avipeF7wfHO7y2kQzRdujEFupaqyvbs/aGCDo8wgg6LBgorBgEEAYI3AwMBMYIO
# ezCCDncGCSqGSIb3DQEHAqCCDmgwgg5kAgEDMQ8wDQYJYIZIAWUDBAIBBQAwgfIG
# CyqGSIb3DQEJEAEEoIHiBIHfMIHcAgEBBglghkiG+mwKAwUwMTANBglghkgBZQME
# AgEFAAQgRZhA2RNoxa/O6Jt3OyxJLplFtCMucke2RBvBX2JWNrACBlsRdW7eBRgT
# MjAxODExMTQxNjE4NDAuMDQxWjAEgAIB9KB2pHQwcjELMAkGA1UEBhMCQ0ExEDAO
# BgNVBAgTB09udGFyaW8xDzANBgNVBAcTBk90dGF3YTEWMBQGA1UEChMNRW50cnVz
# dCwgSW5jLjEoMCYGA1UEAxMfRW50cnVzdCBUaW1lIFN0YW1waW5nIEF1dGhvcml0
# eaCCCe8wggTUMIIDvKADAgECAhEAzXDmaahZIxEAAAAAVZHa3TANBgkqhkiG9w0B
# AQsFADCBsjELMAkGA1UEBhMCVVMxFjAUBgNVBAoTDUVudHJ1c3QsIEluYy4xKDAm
# BgNVBAsTH1NlZSB3d3cuZW50cnVzdC5uZXQvbGVnYWwtdGVybXMxOTA3BgNVBAsT
# MChjKSAyMDE1IEVudHJ1c3QsIEluYy4gLSBmb3IgYXV0aG9yaXplZCB1c2Ugb25s
# eTEmMCQGA1UEAxMdRW50cnVzdCBUaW1lc3RhbXBpbmcgQ0EgLSBUUzEwHhcNMTgw
# NjAxMTU0MzQ3WhcNMjkwOTAxMTYxMzQ3WjByMQswCQYDVQQGEwJDQTEQMA4GA1UE
# CBMHT250YXJpbzEPMA0GA1UEBxMGT3R0YXdhMRYwFAYDVQQKEw1FbnRydXN0LCBJ
# bmMuMSgwJgYDVQQDEx9FbnRydXN0IFRpbWUgU3RhbXBpbmcgQXV0aG9yaXR5MIIB
# IjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAqkcURW9tTf+GlbffWIQd22el
# +8xm29vQQirN/d0uIZh6odz22IDJyKmxMSBHM4PAx+BXxFNnEsoaiSNoaugtvMT9
# fRNx38ssWkBoSx+kkFbaFQCSOM4KdVbvkaHXN7ny60HBr4C4A0PahR6MfQyRFgey
# HY5shRysKcSffA/m5FT1NfAIZr8DGmUDzXQNnwJCGVlxTt1e94evLxZECiq5/JEi
# NZEtzsb4DJTgaemnc2Tr6m8bSNSXIQtyYW4DLoW1k14if/neq1+Sre4xzbRll+FN
# 5f8eIRcZG3o9hiO0WlnVw1L04lnKZ/KvcKA83YiiWzr7awPOXvC6OSucF0oDuwID
# AQABo4IBIjCCAR4wDgYDVR0PAQH/BAQDAgeAMBYGA1UdJQEB/wQMMAoGCCsGAQUF
# BwMIMEEGA1UdIAQ6MDgwNgYKYIZIAYb6bAoDBTAoMCYGCCsGAQUFBwIBFhpodHRw
# Oi8vd3d3LmVudHJ1c3QubmV0L3JwYTAJBgNVHRMEAjAAMDMGCCsGAQUFBwEBBCcw
# JTAjBggrBgEFBQcwAYYXaHR0cDovL29jc3AuZW50cnVzdC5uZXQwMQYDVR0fBCow
# KDAmoCSgIoYgaHR0cDovL2NybC5lbnRydXN0Lm5ldC90czFjYS5jcmwwHwYDVR0j
# BBgwFoAUw8Jx0nvXaAWuOzmbNCUMYgPHV2gwHQYDVR0OBBYEFBogt9wcagoW7dMY
# 5UwcJOPrJsrLMA0GCSqGSIb3DQEBCwUAA4IBAQCxAdm3XIWVt84iEurQMcjAe2cq
# rmrH2wGNLcUXLeM57WT22xB/SqAIx9apH1oON530lzlew8TpzTsW7h5SqHqJ+R2t
# 6pXroZEO997UaRDg3iG3z8EHmDFaoudzPsq+znUQBVTgcWPraOLKdhJIr3eTZ5Go
# cZJ1I8uVPcMevUBi3JYQuxNgroUCk2tYFrRbA5Ifxde9fSKIxhTqBahMsdjkLfhj
# 1wi6VFL6ab2U4F3z3HnvcVZ7kB/VoTfzadEZpeQHSOIJmOdN78m5kQwm77QHZtvB
# leLgkMmMybN2LWom/KXk0AWnPcP0+hx8ktF2tlk0ySLuc2qyXK7PVyoY+CfBMIIF
# EzCCA/ugAwIBAgIMWNoT/wAAAABRzg33MA0GCSqGSIb3DQEBCwUAMIG0MRQwEgYD
# VQQKEwtFbnRydXN0Lm5ldDFAMD4GA1UECxQ3d3d3LmVudHJ1c3QubmV0L0NQU18y
# MDQ4IGluY29ycC4gYnkgcmVmLiAobGltaXRzIGxpYWIuKTElMCMGA1UECxMcKGMp
# IDE5OTkgRW50cnVzdC5uZXQgTGltaXRlZDEzMDEGA1UEAxMqRW50cnVzdC5uZXQg
# Q2VydGlmaWNhdGlvbiBBdXRob3JpdHkgKDIwNDgpMB4XDTE1MDcyMjE5MDI1NFoX
# DTI5MDYyMjE5MzI1NFowgbIxCzAJBgNVBAYTAlVTMRYwFAYDVQQKEw1FbnRydXN0
# LCBJbmMuMSgwJgYDVQQLEx9TZWUgd3d3LmVudHJ1c3QubmV0L2xlZ2FsLXRlcm1z
# MTkwNwYDVQQLEzAoYykgMjAxNSBFbnRydXN0LCBJbmMuIC0gZm9yIGF1dGhvcml6
# ZWQgdXNlIG9ubHkxJjAkBgNVBAMTHUVudHJ1c3QgVGltZXN0YW1waW5nIENBIC0g
# VFMxMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA2SPmFKTofEuFcVj7
# +IHmcotdRsOIAB840Irh1m5WMOWv2mRQfcITOfu9ZrTahPuD0Cgfy3boYFBpm/PO
# TxPiwT7B3xLLMqP4XkQiDsw66Y1JuWB0yN5UPUFeQ18oRqmmt8oQKyK8W01bjBdl
# Eob9LHfVxaCMysKD4EdXfOdwrmJFJzEYCtTApBhVUvdgxgRLs91oMm4QHzQRuBJ4
# ZPHuqeD347EijzRaZcuK9OFFUHTfk5emNObQTDufN0lSp1NOny5nXO2W/KW/dFGI
# 46qOvdmxL19QMBb0UWAia5nL/+FUO7n7RDilCDkjm2lH+jzE0Oeq30ay7PKKGawp
# sjiVdQIDAQABo4IBIzCCAR8wEgYDVR0TAQH/BAgwBgEB/wIBADAOBgNVHQ8BAf8E
# BAMCAQYwOwYDVR0gBDQwMjAwBgRVHSAAMCgwJgYIKwYBBQUHAgEWGmh0dHA6Ly93
# d3cuZW50cnVzdC5uZXQvcnBhMDMGCCsGAQUFBwEBBCcwJTAjBggrBgEFBQcwAYYX
# aHR0cDovL29jc3AuZW50cnVzdC5uZXQwMgYDVR0fBCswKTAnoCWgI4YhaHR0cDov
# L2NybC5lbnRydXN0Lm5ldC8yMDQ4Y2EuY3JsMBMGA1UdJQQMMAoGCCsGAQUFBwMI
# MB0GA1UdDgQWBBTDwnHSe9doBa47OZs0JQxiA8dXaDAfBgNVHSMEGDAWgBRV5IHR
# EYC+2Im5CKMx+aEkCRa5cDANBgkqhkiG9w0BAQsFAAOCAQEAHSTnmnRbqnD8sQ4x
# RdcsAH9mOiugmjSqrGNtifmf3w13/SQj/E+ct2+P8/QftsH91hzEjIhmwWONuld3
# 07gaHshRrcxgNhqHaijqEWXezDwsjHS36FBD08wo6BVsESqfFJUpyQVXtWc26Dyp
# g+9BwSEW0373LRFHZnZgghJpjHZVcw/fL0td6Wwj+Af2tX3WaUWcWH1hLvx4S0NO
# iZFGRCygU6hFofYWWLuRE/JLxd8LwOeuKXq9RbPncDDnNI7revbTtdHeaxOZRrOL
# 0k2TdbXxb7/cACjCJb+856NlNOw/DR2XjPqqiCKkGDXbBY524xDIKY9j0K6sGNna
# xJ9REjGCA2QwggNgAgEBMIHIMIGyMQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50
# cnVzdCwgSW5jLjEoMCYGA1UECxMfU2VlIHd3dy5lbnRydXN0Lm5ldC9sZWdhbC10
# ZXJtczE5MDcGA1UECxMwKGMpIDIwMTUgRW50cnVzdCwgSW5jLiAtIGZvciBhdXRo
# b3JpemVkIHVzZSBvbmx5MSYwJAYDVQQDEx1FbnRydXN0IFRpbWVzdGFtcGluZyBD
# QSAtIFRTMQIRAM1w5mmoWSMRAAAAAFWR2t0wDQYJYIZIAWUDBAIBBQCgggFsMBoG
# CSqGSIb3DQEJAzENBgsqhkiG9w0BCRABBDAvBgkqhkiG9w0BCQQxIgQgpKnBe8P6
# G/lXMk2rM+c810fYBQcoZXepMq3dgdJWsI0wggEbBgsqhkiG9w0BCRACDDGCAQow
# ggEGMIIBAjCB5wQUSdhZxXIaXt9CzQql2YpizJnknNkwgc4wgbikgbUwgbIxCzAJ
# BgNVBAYTAlVTMRYwFAYDVQQKEw1FbnRydXN0LCBJbmMuMSgwJgYDVQQLEx9TZWUg
# d3d3LmVudHJ1c3QubmV0L2xlZ2FsLXRlcm1zMTkwNwYDVQQLEzAoYykgMjAxNSBF
# bnRydXN0LCBJbmMuIC0gZm9yIGF1dGhvcml6ZWQgdXNlIG9ubHkxJjAkBgNVBAMT
# HUVudHJ1c3QgVGltZXN0YW1waW5nIENBIC0gVFMxAhEAzXDmaahZIxEAAAAAVZHa
# 3TAWBBTb6daIuK+CUjlunPOQQbu6MvbVFTANBgkqhkiG9w0BAQsFAASCAQBeFu3z
# fsC1zpHgzYp0/IlsOUhhCIC7IQJXtmYdmfobeZitJAL8S8QAfYfXdC95CZ/sz0Np
# 17S//fZ/jeH3ysE2S4C2REhFNFJmK1yr1E1PGMsJ/ktNwuv2WM+HQhzfs0FfvpMm
# KSGjaWo/Koj6ap3VtlqFnw46PFm5DKFqxDO1fXP/ooAzw+hAxqgfJL7iNvqC4sJP
# INmx7mgZl3l8+8I6v9yTN3NSgEMVb3FDbdT6dspIU78VdiOwLnU71bgAT8SE4XCk
# NtugXtXmKQpR8IZNZ+rE45mcMhPoiCRxbF2TuETu/WxH6J86VXCVA1c8umUwcDyU
# rJlWQsoHkBJa1otY
# SIG # End signature block
