<#

.SYNOPSIS
This Powershell script can be run on a server to gather a file collection for IBM Spectrum Protect (TSM).
The generated files can be uploaded to LiveOpics through the collector for analysis. 

.DESCRIPTION
This script will run queries against the TSM client and produce .out files for analysis.

.EXAMPLE
./LiveOpticsTestTSM.ps1 'C:\Program Files\Tivoli\TSM\baclient' 'Admin' 'MyUniquePassword27' 'C:\Program Files\Tivoli\TSM\baclient\dsm.opt'

.LINK
https://www.liveoptics.com/

#>

#-----------------------------------------------------------------------------
#  Copyright (c) 2017-2018 by Dell Inc. 
# 
#  All rights reserved.  
# 
#-----------------------------------------------------------------------------

param(
    [Parameter(Mandatory=$True,
           helpmessage="Enter the path to the IBM Spectrum Protect client install directory")]
           [string]$directory,
    
    [Parameter(Mandatory=$True,
           helpmessage="Enter the user name to login to IBM Spectrum Protect Server")]
           [string]$tsmusername,
   
    [Parameter(Mandatory=$True,
           helpmessage="Enter the password to login to IBM Spectrum Protect Server")]
           [string]$tsmpassword,
     [Parameter(Mandatory=$True,
           helpmessage="Enter the literal path (including file name) to the dsm.opt file")]
           [string]$dsmoptpath
           )

$scriptVersion="1.0.5";

""
"------------------------------------------------------";
"Live Optics collection script for TSM on Windows";
"Tested with Version 8.1.5.0";
"------------------------------------------------------";
$date=Get-Date -Format o;
"Date: " + $date;
"Version: " + $scriptVersion;
""

#
# UTF-8 output to file without BOM
#
function Out-FileUtf8NoBom {

  [CmdletBinding()]
  param(
    [Parameter(Mandatory=$True, Position=0)] [string] $LiteralPath,
    [switch] $Append,
    [switch] $NoClobber,
    [Parameter(ValueFromPipeline=$True)]
    [string[]]$lines
  )

  Begin
  {
	# Make sure that the .NET framework sees the same working dir. as PS
	# and resolve the input path to a full path.
	[System.IO.Directory]::SetCurrentDirectory($PWD) # Caveat: .NET Core doesn't support [Environment]::CurrentDirectory
	$LiteralPath = [IO.Path]::GetFullPath($LiteralPath)

	# If -NoClobber was specified, throw an exception if the target file already
	# exists.
	if ($NoClobber -and (Test-Path $LiteralPath)) {
		Throw [IO.IOException] "The file '$LiteralPath' already exists."
	}

	try {
		
		# Create a StreamWriter object.
		# Note that we take advantage of the fact that the StreamWriter class by default:
		# - uses UTF-8 encoding
		# - without a BOM.
		$sw = New-Object IO.StreamWriter $LiteralPath, $Append

	}
	catch {
		Write-Error "Failed to create output stream."
		Write-Error $_;
		exit;
    }	
  }
  
  Process { 
	try {
		ForEach ($line in $lines) {
			$sw.WriteLine($line)
		}
	}
	catch {
		$sw.Dispose();
		$sw = $null;
		
		Write-Error "Failed to write to stream."
		Write-Error $_;
		exit;
	}
  }
  
  End {
	if ($sw) {
		$sw.Dispose();
	}
  }
}

#
#Get-Headers - retrieves a comma separated list of Columns
#
function Make-File{
[CmdletBinding()]
    param(
    [Parameter(Mandatory=$True, Position=0)] [string] $TableName
    )
     $storagepath = "$tempDir/LO_TSM_" + $TableName + ".out";
     #Get the Headers
     $columns = dsmadmc "-id=$tsmusername" "-pass=$tsmpassword" "-optfile=$dsmoptpath" "-dataonly=yes" "-DISPL=TAB" "-TAB" "select COLNAME from COLUMNS WHERE tabname = '$TableName' Order By COLNO";
     $columns -join "," | Out-FileUtf8NoBom $storagepath;
     #Append the data to the file 
     & dsmadmc "-id=$tsmusername" "-pass=$tsmpassword" "-optfile=$dsmoptpath" "-dataonly=yes" "-comma" "-DISPL=TAB" "-TAB" "select * from $TableName" |  Out-FileUtf8NoBom -Append $storagepath;

    }


#
# Process Tables - Query DB and generate file
#
function ProcessTables{

    [CmdletBinding()]
    param(
    [Parameter(Mandatory=$True, Position=0)] [string[]] $TableNames
    )
    foreach ($item in $TableNames)
	{
        $shortItem = $item.Trim();
        "Generating file for Table: $shortItem"
        Make-File($shortItem);
	}
}


#
# Create a temporary directory
#
$dt=Get-Date -UFormat %Y%m%d%H%M%S
$tempDir="LiveOpticsTSM" + $dt
new-item -type directory -path $tempDir | Out-Null;

if (!(Test-Path $tempDir)) {
    
    Write-Error "Failed to create directory to hold file output: $tempDir";
    exit;
}

"Temporary directory created: " + $tempDir;
" "
#
# Set the path
#
$env:Path += ";$directory";

#
# Date information - LiveOptics_date.out
#
$LiveOptics_date="$tempDir/LO_TSM_date.out";
$date=Get-Date -Uformat "%a %m/%d/%Y"
$date | Out-FileUtf8NoBom $LiveOptics_date;

#
# Time information
#
$startTime=[System.DateTime]::Now;
$startTime.ToString("hh tt", [System.Globalization.CultureInfo]::InvariantCulture.DateTimeFormat) | Out-FileUtf8NoBom "$tempDir/LO_TSM_time.out";

#
# Timezone information - LiveOptics_timezone.out
#
$tz=(& reg query "HKLM\SYSTEM\CurrentControlSet\Control\TimeZoneInformation");
$LiveOptics_timezone="$tempDir/LO_TSM_timezone.out";
$tz | Out-FileUtf8NoBom $LiveOptics_timezone;

#
# Hostname information - LiveOptics_hostname.out
#
$hostnameFile="$tempDir/LO_TSM_hostname.out";
hostname | Out-FileUtf8NoBom $hostnameFile;

#
# Script version
#
$LiveOptics_scriptVersion="$tempDir/LO_TSM_scriptVersion.out";
$scriptVersion | Out-FileUtf8NoBom $quick_scriptVersion;

#
#Windows version
#
$LiveOptics_osVersion="$tempDir/LO_TSM_windowsversion.out";
[environment]::OSVersion.Version | Out-FileUtf8NoBom $quick_osVersion

#
#Powershell version
#
$LiveOptics_psVersion="$tempDir/LO_TSM_psversion.out";
$PSVersionTable.PSVersion | Out-FileUtf8NoBom $quick_psVersion

#
#Query tables in the database
#

    $SomeTables = "AR_COPYGROUPS", "BU_COPYGROUPS", "OCCUPANCY";
    ProcessTables $SomeTables;

    #Table queries filtered by date

    "Generating file for Table: SUMMARY"
    $columns = dsmadmc "-id=$tsmusername" "-pass=$tsmpassword" "-optfile=$dsmoptpath" "-dataonly=yes" "-DISPL=TAB" "-TAB" "select COLNAME from COLUMNS WHERE tabname = 'SUMMARY' Order By COLNO";
    $columns -join "," | Out-FileUtf8NoBom $tempDir/LO_TSM_SUMMARY.out;
    & dsmadmc "-id=$tsmusername" "-pass=$tsmpassword" "-optfile=$dsmoptpath" "-dataonly=yes" "-comma" "-DISPL=TAB" "-TAB" "select * from SUMMARY WHERE START_TIME >= (CURRENT DATE - 60 DAY) AND ACTIVITY IN ('BACKUP', 'ARCHIVE', 'INCR_DBBACKUP', 'FULL_DBBACKUP', 'RESTORE', 'RETRIEVE')" |  Out-FileUtf8NoBom -Append $tempDir/LO_TSM_SUMMARY.out;

    "Generating file for Table: FILESPACES"
    $columns = dsmadmc "-id=$tsmusername" "-pass=$tsmpassword" "-optfile=$dsmoptpath" "-dataonly=yes" "-DISPL=TAB" "-TAB" "select COLNAME from COLUMNS WHERE tabname = 'FILESPACES' Order By COLNO";
    $columns -join "," | Out-FileUtf8NoBom $tempDir/LO_TSM_FILESPACES.out;
    & dsmadmc "-id=$tsmusername" "-pass=$tsmpassword" "-optfile=$dsmoptpath" "-dataonly=yes" "-comma" "-DISPL=TAB" "-TAB" "select * from FILESPACES WHERE BACKUP_START >= (CURRENT DATE - 60 DAY)" |  Out-FileUtf8NoBom -Append $tempDir/LO_TSM_FILESPACES.out;   

    "Generating file for Version"   
    Add-Content $tempDir/LO_TSM_Version.out "Version, Release, Level, Sublevel";
    & dsmadmc "-id=$tsmusername" "-pass=$tsmpassword" "-optfile=$dsmoptpath" "-dataonly=yes" "-comma" "-DISPL=TAB" "-TAB" "Select Version, Release, Level, Sublevel from STATUS" |  Out-FileUtf8NoBom -Append $tempDir/LO_TSM_Version.out;
" "
"Scan complete. You may now upload your files through the collector."


# SIG # Begin signature block
# MIIb6wYJKoZIhvcNAQcCoIIb3DCCG9gCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCAjFbZ+it3BflVS
# fBfzec6gn7ivkbqKsgrDfkN3Hzk5VqCCCh0wggT4MIID4KADAgECAhBzZpXvvr/L
# YwAAAABVZluYMA0GCSqGSIb3DQEBCwUAMIG0MQswCQYDVQQGEwJVUzEWMBQGA1UE
# ChMNRW50cnVzdCwgSW5jLjEoMCYGA1UECxMfU2VlIHd3dy5lbnRydXN0Lm5ldC9s
# ZWdhbC10ZXJtczE5MDcGA1UECxMwKGMpIDIwMTUgRW50cnVzdCwgSW5jLiAtIGZv
# ciBhdXRob3JpemVkIHVzZSBvbmx5MSgwJgYDVQQDEx9FbnRydXN0IENvZGUgU2ln
# bmluZyBDQSAtIE9WQ1MxMB4XDTE4MDMxNjE3NTgyMloXDTE5MDMxNjE4MjgxM1ow
# WDELMAkGA1UEBhMCVVMxDjAMBgNVBAgTBVRleGFzMRMwEQYDVQQHEwpSb3VuZCBS
# b2NrMREwDwYDVQQKEwhEZWxsIEluYzERMA8GA1UEAxMIRGVsbCBJbmMwggEiMA0G
# CSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQDV0MXSafqENSvohK+axYm2szyWUneb
# qjwuSZyrrR5IK7JrbjXShlM4H23eyalb/urqSkFQ+Ox21Fkkb28aYARakg2a4W5h
# ix5DjOBGyQ0A4hIRiSa9YC0Pl/nMqtmorGv1PYcy5a4MatQkwdi7XMgorvoJYobR
# 99Qs+TENQVyz85UHKhliaEHwk6aPvrNqxKaqANe4PqZEB24ogjDK09XtOQ2R5OXs
# eHk0gB+TqmnkwzK5DxTSwpiuLIolC61l7b+oxgkawa+/6g+O8sLtzUmedqiQUdtt
# qekivkHjAUWtOZ5Ve5bDx5cH7OcAODoLGsbzmX9sTjlqs5QIWMRyOfM1AgMBAAGj
# ggFfMIIBWzAOBgNVHQ8BAf8EBAMCB4AwEwYDVR0lBAwwCgYIKwYBBQUHAwMwagYI
# KwYBBQUHAQEEXjBcMCMGCCsGAQUFBzABhhdodHRwOi8vb2NzcC5lbnRydXN0Lm5l
# dDA1BggrBgEFBQcwAoYpaHR0cDovL2FpYS5lbnRydXN0Lm5ldC9vdmNzMS1jaGFp
# bjI1Ni5jZXIwMQYDVR0fBCowKDAmoCSgIoYgaHR0cDovL2NybC5lbnRydXN0Lm5l
# dC9vdmNzMS5jcmwwSgYDVR0gBEMwQTA2BgpghkgBhvpsCgEDMCgwJgYIKwYBBQUH
# AgEWGmh0dHA6Ly93d3cuZW50cnVzdC5uZXQvcnBhMAcGBWeBDAEEMB8GA1UdIwQY
# MBaAFH4aHxoRdFxkyQwflAGr/YFkLqEsMB0GA1UdDgQWBBT5Z797PfJzX8seCrnX
# L3gauHsnHTAJBgNVHRMEAjAAMA0GCSqGSIb3DQEBCwUAA4IBAQCkZP/Z2dTU30+Y
# fEs5tstA95kLGXwZuhSjVH7W2/Y5rzyFfaRjmHQBTIie74XYP8yax/QIj4oF4S/h
# Z0ufnx33dmNrT0I4olVd6IAME4PqeDUtAxc8AXIqCayJa1NLeSJOWZPMyNoFpb+Q
# Jj4hQ0VJel8wXQZ1oAqxch/zrLEdtb9Zg+iQRa6lKEPDZoyeN4WdNQdam76YBpht
# TTjMKgKGkyo2HXlC5b9e7BJ0b/j97O+xH3TrWioYCA9rh7tsHptSulg7ooEYyMA6
# n2MrCdOENLcBK6hxmdTiQFQ3fkqr12TNoqLrRUAEuzBpvXF/KycAq0AupNZR/FZ/
# TFfvJjZsMIIFHTCCBAWgAwIBAgIMQ8ELHAAAAABR03PaMA0GCSqGSIb3DQEBCwUA
# MIG+MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEoMCYGA1UE
# CxMfU2VlIHd3dy5lbnRydXN0Lm5ldC9sZWdhbC10ZXJtczE5MDcGA1UECxMwKGMp
# IDIwMDkgRW50cnVzdCwgSW5jLiAtIGZvciBhdXRob3JpemVkIHVzZSBvbmx5MTIw
# MAYDVQQDEylFbnRydXN0IFJvb3QgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkgLSBH
# MjAeFw0xNTA2MTAxMzQ2MDVaFw0zMDExMTAxNDE2MDVaMIG0MQswCQYDVQQGEwJV
# UzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEoMCYGA1UECxMfU2VlIHd3dy5lbnRy
# dXN0Lm5ldC9sZWdhbC10ZXJtczE5MDcGA1UECxMwKGMpIDIwMTUgRW50cnVzdCwg
# SW5jLiAtIGZvciBhdXRob3JpemVkIHVzZSBvbmx5MSgwJgYDVQQDEx9FbnRydXN0
# IENvZGUgU2lnbmluZyBDQSAtIE9WQ1MxMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8A
# MIIBCgKCAQEA2IYNoGQKavWOlro7mmhq20RA3vUt/OttRFjPjnLDcC1eWtT5C9gR
# NZfAKfE77heh1kS5vx9KmOAZDrXjWQgdUoygsIooQaShhkBQs03HiEHaNpOnTD1j
# LAPLr40AJRFpUCL2SwDOw7IugYXdI91OTbXwwKh0/OtsYCmYcY9o97cd8/hDi2KV
# MoZPyshMS1P+USAwHv8ARMdecZqlG4l0Sv5TPoTqsaOJnfgtQljYCgZQI3AWHAJ3
# lxg82RB9DHrmBSwaCBZmEjhscR3WZNdWwI02w7i1ALjdL7V6v7Gkbnj6bRJ25Jl3
# EaYk5IK5RnmpsKNJW9NTdEOBdG+h+C4vJQIDAQABo4IBITCCAR0wDgYDVR0PAQH/
# BAQDAgEGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMBIGA1UdEwEB/wQIMAYBAf8CAQAw
# MwYIKwYBBQUHAQEEJzAlMCMGCCsGAQUFBzABhhdodHRwOi8vb2NzcC5lbnRydXN0
# Lm5ldDAwBgNVHR8EKTAnMCWgI6Ahhh9odHRwOi8vY3JsLmVudHJ1c3QubmV0L2cy
# Y2EuY3JsMDsGA1UdIAQ0MDIwMAYEVR0gADAoMCYGCCsGAQUFBwIBFhpodHRwOi8v
# d3d3LmVudHJ1c3QubmV0L3JwYTAdBgNVHQ4EFgQUfhofGhF0XGTJDB+UAav9gWQu
# oSwwHwYDVR0jBBgwFoAUanImetAe733nO2lR1GyNn5ASZqswDQYJKoZIhvcNAQEL
# BQADggEBALd0Z7Q+TCUvYjWBnNJ25afiKHh0w+wZ5RP5tiOUT2KnTOZOcsIkkNB5
# uqjN10TfADkw8SjEcm94rJALLaPqha0UwgsUmT/7I3StqKrMImMyJQErytAWRkJa
# p0RHWB0EQeLRMb6XYTuuBBhsenqBX6si/gjWNY6nDfXsINzcbceidtj7ZqhDAwEh
# qKD2TObUgs7XDQHWM6yQo+CSuxgCdUMCegS02BXtV5E8EueNZvi9FMxGgtb6A8AK
# DXyxgA/roRo2pXUfPNg66ueVgd6sWvudtcwRTip0EdZ/eUJhyiFBYB6k3fHZdLDV
# pegLTIGJ38yGFiUZpRRXlaKKInuxZY0xghEkMIIRIAIBATCByTCBtDELMAkGA1UE
# BhMCVVMxFjAUBgNVBAoTDUVudHJ1c3QsIEluYy4xKDAmBgNVBAsTH1NlZSB3d3cu
# ZW50cnVzdC5uZXQvbGVnYWwtdGVybXMxOTA3BgNVBAsTMChjKSAyMDE1IEVudHJ1
# c3QsIEluYy4gLSBmb3IgYXV0aG9yaXplZCB1c2Ugb25seTEoMCYGA1UEAxMfRW50
# cnVzdCBDb2RlIFNpZ25pbmcgQ0EgLSBPVkNTMQIQc2aV776/y2MAAAAAVWZbmDAN
# BglghkgBZQMEAgEFAKCBmjAZBgkqhkiG9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgor
# BgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIBFTAuBgorBgEEAYI3AgEMMSAwHqAcgBoA
# RABlAGwAbAAgAEkAbgBjAC4AIABUAFMATTAvBgkqhkiG9w0BCQQxIgQghOaccftO
# CalgvvEy99vkPyfYyhmEWq0c8YAuMZ8CxXIwDQYJKoZIhvcNAQEBBQAEggEAPa8s
# sdYxVh8gEJg+qbKIrLbYrjJeVNY5Z7jNDCxUNfH2+6ZFQjCPdigxxkz26savYA0W
# RWGo/iw1u3MwKCOHN6H03vcXcB+06TgrObxg6I9ileaQUzC0ErROVJgI3ax3jRIO
# RnvX7JhPCYjwTAMI3p8snoiWi6UoA8b505fVM3HndwrkLv28VLSnuvmXQyGuo8Ot
# AmOXrE8iWHmZIVgBlwfDHkDCmCZPN8dXVDZjqLiQFQiwFoR41jJRaWJjotUh7rPk
# Uu4p7bZibThK4J06jUgJHSn7OFaRBh2LtjAvQ/mTNpNMrDS0jWqa+QmOcnRfVAeY
# dTdjQUZHW/ACLcD7cqGCDo4wgg6KBgorBgEEAYI3AwMBMYIOejCCDnYGCSqGSIb3
# DQEHAqCCDmcwgg5jAgEDMQ8wDQYJYIZIAWUDBAIBBQAwgfEGCyqGSIb3DQEJEAEE
# oIHhBIHeMIHbAgEBBglghkiG+mwKAwUwMTANBglghkgBZQMEAgEFAAQgL8cq2gCJ
# Ygo4ypRb3FdQwQ7vPKqnhUGFkRQkOqNfZucCBlvcwHLvwhgSMjAxODExMTQxNjE5
# MDguOTNaMASAAgH0oHakdDByMQswCQYDVQQGEwJDQTEQMA4GA1UECBMHT250YXJp
# bzEPMA0GA1UEBxMGT3R0YXdhMRYwFAYDVQQKEw1FbnRydXN0LCBJbmMuMSgwJgYD
# VQQDEx9FbnRydXN0IFRpbWUgU3RhbXBpbmcgQXV0aG9yaXR5oIIJ7zCCBNQwggO8
# oAMCAQICEQCEM3uiibeMugAAAABVkdqHMA0GCSqGSIb3DQEBCwUAMIGyMQswCQYD
# VQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEoMCYGA1UECxMfU2VlIHd3
# dy5lbnRydXN0Lm5ldC9sZWdhbC10ZXJtczE5MDcGA1UECxMwKGMpIDIwMTUgRW50
# cnVzdCwgSW5jLiAtIGZvciBhdXRob3JpemVkIHVzZSBvbmx5MSYwJAYDVQQDEx1F
# bnRydXN0IFRpbWVzdGFtcGluZyBDQSAtIFRTMTAeFw0xODA1MjkxODI2MjdaFw0y
# OTA4MjkxODU2MjdaMHIxCzAJBgNVBAYTAkNBMRAwDgYDVQQIEwdPbnRhcmlvMQ8w
# DQYDVQQHEwZPdHRhd2ExFjAUBgNVBAoTDUVudHJ1c3QsIEluYy4xKDAmBgNVBAMT
# H0VudHJ1c3QgVGltZSBTdGFtcGluZyBBdXRob3JpdHkwggEiMA0GCSqGSIb3DQEB
# AQUAA4IBDwAwggEKAoIBAQCvIqq9ktlWegnjroHKGdbDiwBv7sFPpUtiDtK8S4FN
# msnRH0QVaQ7Rfd9HUj/vbBPYfKvDrEznmUhMmj1lqUZF0c+3zZJLxt67UkLFTEel
# +A7YmuIsJcW6Ds2iBUbLU3VtppJBhiTiaMYolczsn6W3/XGAv/pnVodxRRuutid4
# TtBe2xa6Ure1LrwoVH8QHRGWwTBRZsQv6WDwjiYiaTciDf/eCNYafj2Yw+zO8mK/
# SIOvl6UdY2clIdIbSaQdqrn2aG+0Im6w1+26DqHwDDCfnvPGlbk6sq83MnX+9DkH
# VFIJ1Xn1eXJkHe4A3X7G2+TkQOVNaI1BXR7v9SbVsCYPAgMBAAGjggEiMIIBHjAO
# BgNVHQ8BAf8EBAMCB4AwFgYDVR0lAQH/BAwwCgYIKwYBBQUHAwgwQQYDVR0gBDow
# ODA2BgpghkgBhvpsCgMFMCgwJgYIKwYBBQUHAgEWGmh0dHA6Ly93d3cuZW50cnVz
# dC5uZXQvcnBhMAkGA1UdEwQCMAAwMwYIKwYBBQUHAQEEJzAlMCMGCCsGAQUFBzAB
# hhdodHRwOi8vb2NzcC5lbnRydXN0Lm5ldDAxBgNVHR8EKjAoMCagJKAihiBodHRw
# Oi8vY3JsLmVudHJ1c3QubmV0L3RzMWNhLmNybDAfBgNVHSMEGDAWgBTDwnHSe9do
# Ba47OZs0JQxiA8dXaDAdBgNVHQ4EFgQUvGpj/WguLVXCs5aXuTxzWYDbN9AwDQYJ
# KoZIhvcNAQELBQADggEBALHtlVL4vKiP7TNDNZtsB6R2RIA7y0Ua0J0VjD0MFdVo
# AOY4i6W9idoQbJbjYW8hkNcUqxjcTF/7ueKIOU0F5OHsbmUtVYEzukrNzehbQ+co
# UdYU1+haAgajbH9khnd7PVISaWYQCmvFs4pDGQwC2qqKcQx6fd2U/nlPGnX4BCKQ
# JlyGHdoRWM2PcwHJ5SGIiDsePW5FhjV588946f9ZGIQMCM6zFSVT9V0k45pj2dNS
# 7S8xwmlQd89pracaUGjtYtzviViKdM9cV83H4tkmR0hOxor+VOibGgzBe9BhMK4X
# XiqlaAdJm2d/6/FQUWlkplM1gxnCTzb4OnpID3zlNvUwggUTMIID+6ADAgECAgxY
# 2hP/AAAAAFHODfcwDQYJKoZIhvcNAQELBQAwgbQxFDASBgNVBAoTC0VudHJ1c3Qu
# bmV0MUAwPgYDVQQLFDd3d3cuZW50cnVzdC5uZXQvQ1BTXzIwNDggaW5jb3JwLiBi
# eSByZWYuIChsaW1pdHMgbGlhYi4pMSUwIwYDVQQLExwoYykgMTk5OSBFbnRydXN0
# Lm5ldCBMaW1pdGVkMTMwMQYDVQQDEypFbnRydXN0Lm5ldCBDZXJ0aWZpY2F0aW9u
# IEF1dGhvcml0eSAoMjA0OCkwHhcNMTUwNzIyMTkwMjU0WhcNMjkwNjIyMTkzMjU0
# WjCBsjELMAkGA1UEBhMCVVMxFjAUBgNVBAoTDUVudHJ1c3QsIEluYy4xKDAmBgNV
# BAsTH1NlZSB3d3cuZW50cnVzdC5uZXQvbGVnYWwtdGVybXMxOTA3BgNVBAsTMChj
# KSAyMDE1IEVudHJ1c3QsIEluYy4gLSBmb3IgYXV0aG9yaXplZCB1c2Ugb25seTEm
# MCQGA1UEAxMdRW50cnVzdCBUaW1lc3RhbXBpbmcgQ0EgLSBUUzEwggEiMA0GCSqG
# SIb3DQEBAQUAA4IBDwAwggEKAoIBAQDZI+YUpOh8S4VxWPv4geZyi11Gw4gAHzjQ
# iuHWblYw5a/aZFB9whM5+71mtNqE+4PQKB/LduhgUGmb885PE+LBPsHfEssyo/he
# RCIOzDrpjUm5YHTI3lQ9QV5DXyhGqaa3yhArIrxbTVuMF2UShv0sd9XFoIzKwoPg
# R1d853CuYkUnMRgK1MCkGFVS92DGBEuz3WgybhAfNBG4Enhk8e6p4PfjsSKPNFpl
# y4r04UVQdN+Tl6Y05tBMO583SVKnU06fLmdc7Zb8pb90UYjjqo692bEvX1AwFvRR
# YCJrmcv/4VQ7uftEOKUIOSObaUf6PMTQ56rfRrLs8ooZrCmyOJV1AgMBAAGjggEj
# MIIBHzASBgNVHRMBAf8ECDAGAQH/AgEAMA4GA1UdDwEB/wQEAwIBBjA7BgNVHSAE
# NDAyMDAGBFUdIAAwKDAmBggrBgEFBQcCARYaaHR0cDovL3d3dy5lbnRydXN0Lm5l
# dC9ycGEwMwYIKwYBBQUHAQEEJzAlMCMGCCsGAQUFBzABhhdodHRwOi8vb2NzcC5l
# bnRydXN0Lm5ldDAyBgNVHR8EKzApMCegJaAjhiFodHRwOi8vY3JsLmVudHJ1c3Qu
# bmV0LzIwNDhjYS5jcmwwEwYDVR0lBAwwCgYIKwYBBQUHAwgwHQYDVR0OBBYEFMPC
# cdJ712gFrjs5mzQlDGIDx1doMB8GA1UdIwQYMBaAFFXkgdERgL7YibkIozH5oSQJ
# FrlwMA0GCSqGSIb3DQEBCwUAA4IBAQAdJOeadFuqcPyxDjFF1ywAf2Y6K6CaNKqs
# Y22J+Z/fDXf9JCP8T5y3b4/z9B+2wf3WHMSMiGbBY426V3fTuBoeyFGtzGA2Godq
# KOoRZd7MPCyMdLfoUEPTzCjoFWwRKp8UlSnJBVe1ZzboPKmD70HBIRbTfvctEUdm
# dmCCEmmMdlVzD98vS13pbCP4B/a1fdZpRZxYfWEu/HhLQ06JkUZELKBTqEWh9hZY
# u5ET8kvF3wvA564per1Fs+dwMOc0jut69tO10d5rE5lGs4vSTZN1tfFvv9wAKMIl
# v7zno2U07D8NHZeM+qqIIqQYNdsFjnbjEMgpj2PQrqwY2drEn1ESMYIDZDCCA2AC
# AQEwgcgwgbIxCzAJBgNVBAYTAlVTMRYwFAYDVQQKEw1FbnRydXN0LCBJbmMuMSgw
# JgYDVQQLEx9TZWUgd3d3LmVudHJ1c3QubmV0L2xlZ2FsLXRlcm1zMTkwNwYDVQQL
# EzAoYykgMjAxNSBFbnRydXN0LCBJbmMuIC0gZm9yIGF1dGhvcml6ZWQgdXNlIG9u
# bHkxJjAkBgNVBAMTHUVudHJ1c3QgVGltZXN0YW1waW5nIENBIC0gVFMxAhEAhDN7
# oom3jLoAAAAAVZHahzANBglghkgBZQMEAgEFAKCCAWwwGgYJKoZIhvcNAQkDMQ0G
# CyqGSIb3DQEJEAEEMC8GCSqGSIb3DQEJBDEiBCB1zex4H2JLO8820pZJcQhpUyC1
# UNdX+m+owRnaUpdJYjCCARsGCyqGSIb3DQEJEAIMMYIBCjCCAQYwggECMIHnBBRl
# 6GQ6TafRPOlyWXzceLnOFUjvdjCBzjCBuKSBtTCBsjELMAkGA1UEBhMCVVMxFjAU
# BgNVBAoTDUVudHJ1c3QsIEluYy4xKDAmBgNVBAsTH1NlZSB3d3cuZW50cnVzdC5u
# ZXQvbGVnYWwtdGVybXMxOTA3BgNVBAsTMChjKSAyMDE1IEVudHJ1c3QsIEluYy4g
# LSBmb3IgYXV0aG9yaXplZCB1c2Ugb25seTEmMCQGA1UEAxMdRW50cnVzdCBUaW1l
# c3RhbXBpbmcgQ0EgLSBUUzECEQCEM3uiibeMugAAAABVkdqHMBYEFK7goTCq/sgM
# clPyahA+6nFZNYscMA0GCSqGSIb3DQEBCwUABIIBAJeDkCEyGRSThAJ/wemHr8X7
# zWMyTMTW6jcYdr6HXei2qcWH/t5lhavmb/N++m0Xfw7bnNbVH/BdMYAS7GmXzGyj
# oO/sUUS3+4nPlT+5Hpy/YSFlZhJg2qPuD39QeY0Ce1/odB5ECx+T4c44Ob3rsaO8
# AtJ9V0NUd+s/dZyaxk/yebrp+y70Rm0aYfducHbFKXTc4LZTOWY7QBzB+CsD0ANl
# 0MA8LKnyIvgW9O0GHMMlK6/1BA0nXkTvXM21RLasa2bMtiUVKJA9VygNFRfVElft
# mOayPHrxVVP3pzTrPW970cb4MwAqQ8Ov+F4mYY82DlTlumslXsLQxXneOPfLv8U=
# SIG # End signature block
